/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.gui.tools;

import com.net2plan.internal.plugins.IGUIModule;
import com.net2plan.gui.utils.AdvancedJTable;
import com.net2plan.gui.utils.CellRenderers;
import com.net2plan.gui.utils.CellRenderers.NumberCellRenderer;
import com.net2plan.gui.utils.ClassAwareTableModel;
import com.net2plan.gui.utils.ColumnHeaderToolTips;
import com.net2plan.gui.utils.CurrentAndPlannedStateTableSorter;
import com.net2plan.gui.utils.CurrentAndPlannedStateTableSorter.CurrentAndPlannedStateTableCellValue;
import com.net2plan.gui.utils.FixedColumnDecorator;
import com.net2plan.gui.utils.FullScrollPaneLayout;
import com.net2plan.gui.utils.ParamValueTable;
import com.net2plan.gui.utils.ParameterValueDescriptionPanel;
import com.net2plan.gui.utils.PopupMenuNetPlan;
import com.net2plan.gui.utils.ProportionalResizeJSplitPaneListener;
import com.net2plan.gui.utils.ReportBrowser;
import com.net2plan.gui.utils.RunnableSelector;
import com.net2plan.gui.utils.StringLabeller;
import com.net2plan.gui.utils.TabIcon;
import com.net2plan.gui.utils.TableCursorNavigation;
import com.net2plan.gui.utils.ThreadExecutionController;
import com.net2plan.gui.utils.WiderJComboBox;
import com.net2plan.gui.utils.topology.INetworkCallback;
import com.net2plan.gui.utils.topology.ITopologyCanvasPlugin;
import com.net2plan.gui.utils.topology.jung.JUNGCanvas;
import com.net2plan.gui.utils.topology.TopologyPanel;
import com.net2plan.gui.utils.topology.jung.AddLinkGraphPlugin;
import com.net2plan.gui.utils.topology.jung.MoveNodePlugin;
import com.net2plan.gui.utils.topology.jung.PanGraphPlugin;
import com.net2plan.gui.utils.topology.jung.PopupEditGraphPlugin;
import com.net2plan.interfaces.networkDesign.Configuration;
import com.net2plan.interfaces.networkDesign.IReport;
import com.net2plan.interfaces.networkDesign.Net2PlanException;
import com.net2plan.interfaces.networkDesign.NetPlan;
import com.net2plan.internal.Constants;
import com.net2plan.internal.Constants.NetworkElementType;
import com.net2plan.internal.ErrorHandling;
import com.net2plan.internal.SystemUtils;
import com.net2plan.libraries.GraphTheoryMetrics;
import com.net2plan.libraries.GraphUtils;
import com.net2plan.libraries.NetworkPerformanceMetrics;
import com.net2plan.libraries.SRGUtils;
import com.net2plan.libraries.TrafficComputationEngine;
import com.net2plan.utils.ClassLoaderUtils;
import com.net2plan.utils.CollectionUtils;
import com.net2plan.utils.Constants.RoutingCycleType;
import com.net2plan.utils.Constants.RoutingType;
import com.net2plan.utils.DoubleUtils;
import com.net2plan.utils.IntUtils;
import com.net2plan.utils.LongUtils;
import com.net2plan.utils.Pair;
import com.net2plan.utils.StringUtils;
import com.net2plan.utils.Triple;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.event.InputEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.io.Closeable;
import java.io.File;
import java.util.Arrays;
import java.util.EnumMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.RowFilter;
import javax.swing.ScrollPaneLayout;
import javax.swing.SwingUtilities;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import net.miginfocom.swing.MigLayout;

/**
 * Template for any tool requiring network visualization.
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @since 0.3.0
 */
public abstract class IGUINetworkViewer extends IGUIModule implements INetworkCallback, ThreadExecutionController.IThreadExecutionHandler
{
	/**
	 * Reference to the topology panel.
	 * 
	 * @since 0.3.0
	 */
	protected TopologyPanel topologyPanel;
	
	/**
	 * Reference to the popup menu in the topology panel.
	 * 
	 * @since 0.3.0
	 */
	protected ITopologyCanvasPlugin popupPlugin;
	
	private JPanel leftPane;
	private JSplitPane reportPane;
	private JTabbedPane rightPane, netPlanView, reportContainer;
	private RunnableSelector reportSelector;
	private ThreadExecutionController reportController;
	private int viewNetPlanTabIndex;
	private JButton forceUpdate;
	private JRadioButton sourceRoutingActivated, hopByHopRoutingActivated;
	private ButtonGroup routingSchemes;

	private JComboBox layerChooser;
	private JPanel layerChooserPane, layerMetricsInfo;
	private ParamValueTable[] layerSummaryTables;
	private Map<NetworkElementType, JTable> netPlanViewTable;
	private Map<NetworkElementType, FixedColumnDecorator> netPlanViewTableDecorator;
	private Map<NetworkElementType, JComponent> netPlanViewTableComponent;

	private JTextField txt_layerName, txt_networkName, txt_layerLinkCapacityUnits, txt_layerDemandTrafficUnits, txt_numLayers, txt_numNodes, txt_numSRGs;
	private JTextArea txt_layerDescription, txt_networkDescription;
	private JCheckBox showInitialPlan;

	private final static Map<NetworkElementType, String> netPlanViewTabName;
	private final static Map<NetworkElementType, String[]> netPlanViewTableHeader;
	private final static Map<NetworkElementType, String[]> netPlanViewTableTips;
	private final static String[] layerSummaryTableHeader = StringUtils.arrayOf("Metric", "Value");
	
	private final static String[] attributeTableHeader = StringUtils.arrayOf("Attribute", "Value");
	private final static String[] attributeTableTips = attributeTableHeader;
	private JTable networkAttributeTable, layerAttributeTable;
	
	private NetPlan currentNetPlan, initialNetPlan;
	private boolean isEditable;
	
	static
	{
		netPlanViewTabName = new EnumMap<NetworkElementType, String>(NetworkElementType.class);
		netPlanViewTabName.put(NetworkElementType.NODE, "Nodes");
		netPlanViewTabName.put(NetworkElementType.LINK, "Links");
		netPlanViewTabName.put(NetworkElementType.DEMAND, "Demands");
		netPlanViewTabName.put(NetworkElementType.ROUTE, "Routes");
		netPlanViewTabName.put(NetworkElementType.FORWARDING_RULE, "Forwarding rules");
		netPlanViewTabName.put(NetworkElementType.PROTECTION_SEGMENT, "Protection segments");
		netPlanViewTabName.put(NetworkElementType.SRG, "Shared-risk groups");
		netPlanViewTabName.put(NetworkElementType.NETWORK, "Network");
		netPlanViewTabName.put(NetworkElementType.LAYER, "Layer");
		
		netPlanViewTableHeader = new EnumMap<NetworkElementType, String[]>(NetworkElementType.class);
		netPlanViewTableHeader.put(NetworkElementType.NODE, StringUtils.arrayOf("Id", "Show/Hide", "Name", "State", "xCoord", "yCoord", "Ingress traffic", "Egress traffic", "Traversing traffic", "Total traffic", "SRGs", "Attributes"));
		netPlanViewTableHeader.put(NetworkElementType.LINK, StringUtils.arrayOf("Id", "Show/Hide", "Origin node", "Destination node", "State", "Coupled to demand", "Capacity", "Carried traffic", "Reserved for protection", "Utilization", "Utilization (w.o. protection)", "Is bottleneck?", "Length (km)", "Propagation speed (km/s)", "Propagation delay (ms)", "# Routes", "# Segments", "# Forwarding rules", "SRGs", "Attributes"));
		netPlanViewTableHeader.put(NetworkElementType.DEMAND, StringUtils.arrayOf("Id", "Ingress node", "Egress node", "Coupled to link", "Offered traffic", "Carried traffic", "% Lost traffic", "Routing cycles", "Bifurcated", "# Routes", "Attributes"));
		netPlanViewTableHeader.put(NetworkElementType.ROUTE, StringUtils.arrayOf("Id", "Demand", "Ingress node", "Egress node", "Demand offered traffic", "Carried traffic", "Occupied capacity", "Sequence of links", "Sequence of nodes", "Number of hops", "Length (km)", "Propagation delay (ms)", "Bottleneck utilization", "Backup segments", "Attributes"));
		netPlanViewTableHeader.put(NetworkElementType.FORWARDING_RULE, StringUtils.arrayOf("Node", "Demand", "Outgoing link", "Splitting ratio", "Carried traffic", "Attributes"));
		netPlanViewTableHeader.put(NetworkElementType.PROTECTION_SEGMENT, StringUtils.arrayOf("Id", "Origin node", "Destination node", "Reserved capacity", "Carried traffic", "Sequence of links", "Sequence of nodes", "Number of hops", "Length (km)", "Propagation delay (ms)", "Dedicated/Shared", "# Routes", "Attributes"));
		netPlanViewTableHeader.put(NetworkElementType.SRG, StringUtils.arrayOf("Id", "MTTF (hours)", "MTTR (hours)", "Nodes", "Links", "# Affected routes", "# Affected protection segments", "Attributes"));
		netPlanViewTableHeader.put(NetworkElementType.NETWORK, StringUtils.arrayOf("Attribute", "Value"));
		netPlanViewTableHeader.put(NetworkElementType.LAYER, StringUtils.arrayOf("Id", "Name", "Routing type", "Number of links", "Number of demands", "Number of routes", "Number of forwarding rules", "Number of protection segments", "Description", "Link capacity units name", "Demand traffic units name", "Attributes"));
		
		netPlanViewTableTips = new EnumMap<NetworkElementType, String[]>(NetworkElementType.class);
		netPlanViewTableTips.put(NetworkElementType.NODE, StringUtils.arrayOf("Node identifier", "Indicates whether or not the node is visible in the topology canvas", "Node name", "Indicates whether the node is in up/down state", "Coordinate along x-axis (i.e. longitude)", "Coordinate along y-axis (i.e. latitude)", "Total traffic entering to the network from this node", "Total traffic leaving the network from this node", "Traffic entering the node, but not targeted to it", "Total traffic", "SRGs including this node", "Node-specific attributes"));
		netPlanViewTableTips.put(NetworkElementType.LINK, StringUtils.arrayOf("Link identifier", "Indicates whether or not the link is visible in the topology canvas (if some of the end-nodes is hidden, this link will become hidden, even though the link is set as visible)", "Origin node", "Destination node", "Indicates whether the link is in up/down state", "Indicates the coupled lower layer demand, if any, or empty", "Capacity", "Carried traffic", "Reserved for protection", "Utilization", "Utilization excluding reserved capacity for protection", "Indicates whether this link has the highest utilization", "Length (km)", "Propagation speed (km/s)", "Propagation delay (ms)", "Number of routes traversing the link", "Number of protection segments traversing the link", "Number of forwarding rules for this link", "SRGs including this link", "Link-specific attributes"));
		netPlanViewTableTips.put(NetworkElementType.DEMAND, StringUtils.arrayOf("Demand identifier", "Ingress node", "Egress node", "Indicates the coupled upper layer link, if any, or empty", "Offered traffic by the demand", "Carried traffic by routes carrying traffic from the demand", "Percentage of lost traffic from the offered", "Indicates whether there are routing cycles: loopless (no cycle in some route), open cycles (traffic reaches egress node after some cycles in some route), closed cycles (traffic does not reach the egress node in some route)", "Indicates whether the demand has more than one associated route carrying traffic", "Number of associated routes", "Demand-specific attributes"));
		netPlanViewTableTips.put(NetworkElementType.ROUTE, StringUtils.arrayOf("Route identifier", "Demand", "Ingress node", "Egress node", "Demand offered traffic", "Carried traffic", "Occupied capacity", "Sequence of links", "Sequence of nodes", "Number of hops", "Total route length", "Propagation delay (ms)", "Highest utilization among all traversed links", "Candidate protection segments for this route", "Route-specific attributes"));
		netPlanViewTableTips.put(NetworkElementType.FORWARDING_RULE, StringUtils.arrayOf("Node where the forwarding rule is installed", "Demand", "Outgoing link", "Percentage of the traffic entering the node going through the outgoing link", "Carried traffic in this link for the demand", "Attributes"));
		netPlanViewTableTips.put(NetworkElementType.PROTECTION_SEGMENT, StringUtils.arrayOf("Protection segment identifier", "Origin node", "Destination node", "Reserved capacity for the segment", "Carried traffic by this segment", "Sequence of links", "Sequence of nodes", "Number of hops", "Length (km)", "Propagation delay (ms)", "Dedicated/Shared", "# Routes", "Attributes"));
		netPlanViewTableTips.put(NetworkElementType.SRG, StringUtils.arrayOf("SRG identifier", "Mean time to fail", "Mean time to repair", "Nodes included into the shared-risk group", "Links included into the shared-risk group", "# Affected routes", "# Affected protection segments", "Attributes"));
		netPlanViewTableTips.put(NetworkElementType.NETWORK, StringUtils.arrayOf("Attribute", "Value"));
		netPlanViewTableTips.put(NetworkElementType.LAYER, StringUtils.arrayOf("Id", "Name", "Routing type", "Number of links", "Number of demands", "Number of routes", "Number of forwarding rules", "Number of protection segments", "Description", "Link capacity units name", "Demand traffic units name", "Attributes"));
	}

	/**
	 * Constructor that allows set a title for the tool in the top section of the panel.
	 * 
	 * @param title Title of the tool (null or empty means no title)
	 * @since 0.3.0
	 */
	public IGUINetworkViewer(String title)
	{
		super(title);
	}
	
	@Override
	public void addLink(long originNodeId, long destinationNodeId)
	{
		if (!isEditable) throw new UnsupportedOperationException("Not supported");
		
		NetPlan netPlan = getDesign();
		long linkId = netPlan.addLink(originNodeId, destinationNodeId, 0, 0, null);
		getTopologyPanel().getCanvas().addLink(linkId, originNodeId, destinationNodeId);
		getTopologyPanel().getCanvas().refresh();
		updateNetPlanView();
	}

	@Override
	public void addNode(Point2D pos)
	{
		if (!isEditable) throw new UnsupportedOperationException("Not supported");
		
		NetPlan netPlan = getDesign();
		long nodeId = netPlan.getNodeNextId();
		netPlan.addNode(pos.getX(), pos.getY(), "Node " + nodeId, null);
		getTopologyPanel().getCanvas().addNode(nodeId, pos, null);
		getTopologyPanel().getCanvas().refresh();
		updateNetPlanView();
	}
	
	@Override
	public void configure(JPanel contentPane)
	{
		topologyPanel = new TopologyPanel(this, JUNGCanvas.class);
		topologyPanel.setBorder(BorderFactory.createTitledBorder(new LineBorder(Color.BLACK), "Network topology"));
		topologyPanel.setAllowLoadTrafficDemand(allowLoadTrafficDemands());
		configureTopologyPanel();
		configureLayerChooser();

		addKeyCombinationAction("Load design", new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				getTopologyPanel().loadDesign();
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK));

//		addKeyCombinationAction("Show previous layer", new AbstractAction()
//		{
//			@Override
//			public void actionPerformed(ActionEvent e)
//			{
//				int currentIndex = layerChooser.getSelectedIndex();
//				if (currentIndex < layerChooser.getItemCount() - 1) layerChooser.setSelectedIndex(currentIndex + 1);
//			}
//		}, KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_DOWN, InputEvent.CTRL_DOWN_MASK));
//
//		addKeyCombinationAction("Show next layer", new AbstractAction()
//		{
//			@Override
//			public void actionPerformed(ActionEvent e)
//			{
//				int currentIndex = layerChooser.getSelectedIndex();
//				if (currentIndex > 0) layerChooser.setSelectedIndex(currentIndex - 1);
//			}
//		}, KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_UP, InputEvent.CTRL_DOWN_MASK));

		addKeyCombinationAction("Save design", new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				getTopologyPanel().saveDesign();
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK));

		addKeyCombinationAction("Zoom in", new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				getTopologyPanel().zoomIn();
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_ADD, InputEvent.CTRL_DOWN_MASK), KeyStroke.getKeyStroke(KeyEvent.VK_PLUS, InputEvent.CTRL_DOWN_MASK));

		addKeyCombinationAction("Zoom out", new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				getTopologyPanel().zoomOut();
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_SUBTRACT, InputEvent.CTRL_DOWN_MASK), KeyStroke.getKeyStroke(KeyEvent.VK_MINUS, InputEvent.CTRL_DOWN_MASK));

		addKeyCombinationAction("Zoom all", new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				getTopologyPanel().zoomAll();
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_MULTIPLY, InputEvent.CTRL_DOWN_MASK));

		addKeyCombinationAction("Take snapshot", new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				getTopologyPanel().takeSnapshot();
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_F12, InputEvent.CTRL_DOWN_MASK));

		if (allowLoadTrafficDemands())
		{
			addKeyCombinationAction("Load traffic demands", new AbstractAction()
			{
				@Override
				public void actionPerformed(ActionEvent e)
				{
					getTopologyPanel().loadTrafficDemands();
				}
			}, KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_DOWN_MASK));
		}

		leftPane = new JPanel(new BorderLayout());
		JPanel logSection = configureLeftBottomPanel();
		if (logSection == null)
		{
			leftPane.add(topologyPanel, BorderLayout.CENTER);
		}
		else
		{
			JSplitPane splitPaneTopology = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			splitPaneTopology.setTopComponent(topologyPanel);
			splitPaneTopology.setBottomComponent(logSection);
			splitPaneTopology.setResizeWeight(0.8);
			splitPaneTopology.addPropertyChangeListener(new ProportionalResizeJSplitPaneListener());
			splitPaneTopology.setBorder(new LineBorder(contentPane.getBackground()));
			leftPane.add(splitPaneTopology, BorderLayout.CENTER);
		}
		
		rightPane = new JTabbedPane();
//		rightPane.setFocusable(false);

		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setLeftComponent(leftPane);
		splitPane.setRightComponent(rightPane);
		splitPane.setResizeWeight(0.5);
		splitPane.addPropertyChangeListener(new ProportionalResizeJSplitPaneListener());

		splitPane.setBorder(BorderFactory.createEmptyBorder());
		contentPane.add(splitPane, "grow");
		
		configureNetPlanView();
		configureReportPane();
		loadDesign(new NetPlan());
		
		addKeyCombinationAction("Resets the tool", new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				reset();
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK));

		addKeyCombinationAction("Outputs current design to console", new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				System.out.println(getDesign().toString());
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_F11, InputEvent.CTRL_DOWN_MASK));
		
		for(int tabId = 0; tabId <= 8; tabId++)
		{
			final int key;
			switch(tabId)
			{
				case 0:
					key = KeyEvent.VK_1;
					break;

				case 1:
					key = KeyEvent.VK_2;
					break;

				case 2:
					key = KeyEvent.VK_3;
					break;

				case 3:
					key = KeyEvent.VK_4;
					break;

				case 4:
					key = KeyEvent.VK_5;
					break;

				case 5:
					key = KeyEvent.VK_6;
					break;

				case 6:
					key = KeyEvent.VK_7;
					break;

				case 7:
					key = KeyEvent.VK_8;
					break;

				case 8:
					key = KeyEvent.VK_9;
					break;

				default:
					throw new RuntimeException("Bad");
			}
	
			addKeyCombinationAction("Open right tab " + tabId, new SwitchTabAction(tabId), KeyStroke.getKeyStroke(key, InputEvent.CTRL_DOWN_MASK));
		}		
	}
	
	@Override
	public Object execute(ThreadExecutionController controller)
	{
		if (controller == reportController)
		{
			Triple<File, String, Class> report = reportSelector.getRunnable();
			Map<String, String> reportParameters = reportSelector.getRunnableParameters();
			Map<String, String> net2planParameters = Configuration.getOptions();
			IReport instance = ClassLoaderUtils.getInstance(report.getFirst(), report.getSecond(), IReport.class);
			Pair<String, ? extends JPanel> aux = Pair.of(instance.getTitle(), new ReportBrowser(instance.executeReport(getDesign().copy(), reportParameters, net2planParameters)));
			try { ((Closeable) instance.getClass().getClassLoader()).close(); }
			catch (Throwable e) { }

			return aux;
		}
		else
		{
			throw new RuntimeException("Bad");
		}
	}

	@Override
	public void executionFailed(ThreadExecutionController controller)
	{
		if (controller == reportController)
		{
			ErrorHandling.showErrorDialog("Error executing report");
		}
		else
		{
			ErrorHandling.showErrorDialog("Bad");
		}
	}

	@Override
	public void executionFinished(ThreadExecutionController controller, Object out)
	{
		if (controller == reportController)
		{
			Pair<String, ? extends JPanel> aux = (Pair<String, ? extends JPanel>) out;
			reportContainer.addTab(aux.getFirst(), new TabIcon(TabIcon.IconType.TIMES_SIGN), aux.getSecond());
			reportContainer.setSelectedIndex(reportContainer.getTabCount() - 1);
		}
		else
		{
			ErrorHandling.showErrorDialog("Bad");
		}
	}
	
	@Override
	public NetPlan getDesign()
	{
		return currentNetPlan;
	}
	
	@Override
	public NetPlan getInitialDesign()
	{
		return initialNetPlan;
	}

	@Override
	public boolean isEditable()
	{
		return isEditable;
	}
	
	@Override
	public void loadDesign(NetPlan netPlan)
	{
		setNetPlan(netPlan);
		updateLayerChooser();
		topologyPanel.getCanvas().zoomAll();
	}

	@Override
	public void loadTrafficDemands(NetPlan demands)
	{
		if (!demands.hasDemands()) throw new Net2PlanException("Selected file doesn't contain a demand set");
		
		NetPlan netPlan = getDesign();
		if (netPlan.hasDemands())
		{
			int result = JOptionPane.showConfirmDialog(null, "Current network structure contains a demand set. Overwrite?", "Loading demand set", JOptionPane.YES_NO_OPTION);
			if (result != JOptionPane.YES_OPTION) return;
		}

		NetPlan aux_netPlan = netPlan.copy();
		try
		{
			netPlan.removeAllDemands();
			Set<Long> demandIds = demands.getDemandIds();
			for(long demandId : demandIds)
			{
				Pair<Long, Long> ingressEgressNodePair = demands.getDemandIngressEgressNodePair(demandId);
				long ingressNodeId = ingressEgressNodePair.getFirst();
				long egressNodeId = ingressEgressNodePair.getSecond();
				double h_d = demands.getDemandOfferedTraffic(demandId);
				Map<String, String> demandAttributes = demands.getDemandAttributeMap(demandId);
				netPlan.addDemand(ingressNodeId, egressNodeId, h_d, demandAttributes);
			}

			updateNetPlanView();
		}
		catch (Throwable ex)
		{
			getDesign().assignFrom(aux_netPlan);
			throw new RuntimeException(ex);
		}
	}
	
	@Override
	public void moveNode(long nodeId, Point2D pos)
	{
		if (!isEditable) throw new UnsupportedOperationException("Not supported");
		
		TableModel nodeTableModel = netPlanViewTable.get(NetworkElementType.NODE).getModel();
		int numRows = nodeTableModel.getRowCount();
		for(int row = 0; row < numRows; row++)
		{
			if ((long) nodeTableModel.getValueAt(row, 0) == nodeId)
			{
				nodeTableModel.setValueAt(pos.getX(), row, 4);
				nodeTableModel.setValueAt(pos.getY(), row, 5);
			}
		}
	}

	@Override
	public void removeLink(long linkId)
	{
		if (!isEditable) throw new UnsupportedOperationException("Not supported");
		
		NetPlan netPlan = getDesign();
		netPlan.removeLink(linkId);
		getTopologyPanel().getCanvas().removeLink(linkId);
		getTopologyPanel().getCanvas().refresh();
		updateNetPlanView();
	}

	@Override
	public void removeNode(long nodeId)
	{
		if (!isEditable) throw new UnsupportedOperationException("Not supported");
		
		NetPlan netPlan = getDesign();
		netPlan.removeNode(nodeId);
		getTopologyPanel().getCanvas().removeNode(nodeId);
		getTopologyPanel().getCanvas().refresh();
		updateNetPlanView();
	}

	@Override
	public void reset()
	{
		try
		{
			boolean reset = askForReset();
			if (!reset) return;

			reset_internal();
			reportSelector.reset();
			reportContainer.removeAll();
		}
		catch (Throwable ex)
		{
			ErrorHandling.addErrorOrException(ex, IGUINetworkViewer.class);
			ErrorHandling.showErrorDialog("Unable to reset");
		}
	}
	
	@Override
	public void resetView()
	{
		topologyPanel.getCanvas().resetPickedState();
		for (JTable table : netPlanViewTable.values())
			table.clearSelection();
	}
	
	@Override
	public void showDemand(long demandId)
	{
		showDemand(getDesign().getLayerDefaultId(), demandId);
	}
	
	@Override
	public void showDemand(long layerId, long demandId)
	{	
		selectNetPlanViewItem(layerId, NetworkElementType.DEMAND, demandId);
		
		NetPlan netPlan = getDesign();
		Pair<Long, Long> ingressEgressNodePair = netPlan.getDemandIngressEgressNodePair(demandId);
		long ingressNodeId = ingressEgressNodePair.getFirst();
		long egressNodeId = ingressEgressNodePair.getSecond();
		List<Long> nodeIds = new LinkedList<Long>();
		nodeIds.add(ingressNodeId);
		nodeIds.add(egressNodeId);
		
		RoutingType routingType = netPlan.getRoutingType();
		if (routingType == RoutingType.HOP_BY_HOP_ROUTING)
		{
			Set<Pair<Long, Long>> forwardingRules = netPlan.getDemandForwardingRules(demandId);
			Set<Long> linkIds = new LinkedHashSet<Long>();
			for(Pair<Long, Long> forwardingRule : forwardingRules)
			{
				double x_de = netPlan.getForwardingRuleCarriedTraffic(forwardingRule);
				if (x_de > 0) linkIds.add(forwardingRule.getSecond());
			}

			topologyPanel.getCanvas().showNodesAndLinks(nodeIds, linkIds);
		}
		else
		{
			topologyPanel.getCanvas().showNodes(nodeIds);
		}
		
		topologyPanel.getCanvas().refresh();
	}
	
	@Override
	public void showForwardingRule(Pair<Long, Long> forwardingRule)
	{
		showForwardingRule(getDesign().getLayerDefaultId(), forwardingRule);
	}
	
	@Override
	public void showForwardingRule(long layerId, Pair<Long, Long> forwardingRule)
	{	
		selectNetPlanViewItem(layerId, NetworkElementType.FORWARDING_RULE, forwardingRule);
		
		NetPlan netPlan = getDesign();
		
		long demandId = forwardingRule.getFirst();
		Pair<Long, Long> ingressEgressNodePair = netPlan.getDemandIngressEgressNodePair(demandId);
		List<Long> nodeIds = new LinkedList<Long>();
		nodeIds.add(ingressEgressNodePair.getFirst());
		nodeIds.add(ingressEgressNodePair.getSecond());
		
		long linkId = forwardingRule.getSecond();
		Set<Long> linkIds = new LinkedHashSet<Long>();
		linkIds.add(linkId);
		
		topologyPanel.getCanvas().showNodesAndLinks(nodeIds, linkIds);
		topologyPanel.getCanvas().refresh();
	}

	@Override
	public void showLink(long linkId)
	{
		showLink(getDesign().getLayerDefaultId(), linkId);
	}
	
	@Override
	public void showLink(long layerId, long linkId)
	{
		selectNetPlanViewItem(layerId, NetworkElementType.LINK, linkId);
		
		topologyPanel.getCanvas().showLink(linkId);
		topologyPanel.getCanvas().refresh();
	}

	@Override
	public void showNode(long nodeId)
	{
		showNode(getDesign().getLayerDefaultId(), nodeId);
	}
	
	@Override
	public void showNode(long layerId, long nodeId)
	{
		selectNetPlanViewItem(layerId, NetworkElementType.NODE, nodeId);
		
		topologyPanel.getCanvas().showNode(nodeId);
		topologyPanel.getCanvas().refresh();
	}
	
	@Override
	public void showProtectionSegment(long segmentId)
	{
		showProtectionSegment(getDesign().getLayerDefaultId(), segmentId);
	}
	
	@Override
	public void showProtectionSegment(long layerId, long segmentId)
	{
		selectNetPlanViewItem(layerId, NetworkElementType.PROTECTION_SEGMENT, segmentId);
		
		List<Long> seqLinks = getDesign().getProtectionSegmentSequenceOfLinks(segmentId);
		topologyPanel.getCanvas().showRoute(seqLinks);
		topologyPanel.getCanvas().refresh();
	}

	@Override
	public void showRoute(long routeId)
	{
		showRoute(getDesign().getLayerDefaultId(), routeId);
	}
	
	@Override
	public void showRoute(long layerId, long routeId)
	{
		selectNetPlanViewItem(layerId, NetworkElementType.ROUTE, routeId);
		
		NetPlan currentState = getDesign();
		NetPlan initialState = getInitialDesign();
		Set<Long> pri_seqLinks = new LinkedHashSet<Long>(currentState.convertSequenceOfLinksAndProtectionSegmentsToSequenceOfLinks(currentState.getRouteSequenceOfLinks(routeId)));
		Set<Long> sec_seqLinks = new LinkedHashSet<Long>();

		if (allowShowInitialNetPlan())
		{
			if (initialState.isLayerActive(layerId) && initialState.isRouteActive(layerId, routeId))
				sec_seqLinks = new LinkedHashSet<Long>(initialState.convertSequenceOfLinksAndProtectionSegmentsToSequenceOfLinks(layerId, initialState.getRouteSequenceOfLinks(layerId, routeId)));
			
			Iterator<Long> linkIt = sec_seqLinks.iterator();
			while(linkIt.hasNext())
			{
				long linkId = linkIt.next();
				if (!currentState.isLinkActive(linkId)) linkIt.remove();
			}
		}
		
		topologyPanel.getCanvas().showRoutes(pri_seqLinks, sec_seqLinks);
		topologyPanel.getCanvas().refresh();
	}

	@Override
	public void showSRG(long srgId)
	{
		showSRG(getDesign().getLayerDefaultId(), srgId);
	}
	
	@Override
	public void showSRG(long layerId, long srgId)
	{
		selectNetPlanViewItem(layerId, NetworkElementType.SRG, srgId);
		
		Set<Long> nodeIds_thisSRG = getDesign().getSRGNodes(srgId);
		Set<Long> linkIds_thisSRG_thisLayer = getDesign().getSRGLinks(srgId);
		topologyPanel.getCanvas().showNodesAndLinks(nodeIds_thisSRG, linkIds_thisSRG_thisLayer);
		topologyPanel.getCanvas().refresh();
	}

	@Override
	public final void updateLayerChooser()
	{
		ActionListener[] al = layerChooser.getActionListeners();
		for (ActionListener a : al) layerChooser.removeActionListener(a);

		layerChooser.removeAllItems();
		
		NetPlan currentState = getDesign();
		Set<Long> layerIds = currentState.getLayerIds();
		Map<Long, Integer> layerId2LinearIndex = CollectionUtils.convertId2LinearIndexMap(layerIds);
		for(long layerId : layerIds)
		{
			String layerName_thisLayer = currentState.getLayerName(layerId);
			layerName_thisLayer = "Layer " + layerId + (layerName_thisLayer.equals("") ? "" : ": " + layerName_thisLayer);
			layerChooser.addItem(StringLabeller.of(layerId, layerName_thisLayer));
		}
		
		for (ActionListener a : al) layerChooser.addActionListener(a);

		long defaultLayerId = currentState.getLayerDefaultId();
		layerChooser.setSelectedIndex(layerId2LinearIndex.get(defaultLayerId));

		if (layerChooser.getItemCount() > 1) leftPane.add(layerChooserPane, BorderLayout.NORTH);
		else leftPane.remove(layerChooserPane);

		leftPane.revalidate();
	}
	
	@Override
	public synchronized void updateNetPlanView()
	{
		updateWarnings();
		
		/* Load current network state */
		NetPlan currentState = getDesign();
		long layerId = currentState.getLayerDefaultId();
		
		for (Entry<NetworkElementType, JTable> entry : netPlanViewTable.entrySet())
		{
			NetworkElementType elementType = entry.getKey();
			JTable table = entry.getValue();
			table.setEnabled(false);

			String[] header = netPlanViewTableHeader.get(elementType);
			((DefaultTableModel)table.getModel()).setDataVector(new Object[1][header.length], header);
		}
		
		RoutingType routingType = currentState.getRoutingType();

		Component selectedTab = netPlanView.getSelectedComponent();
		netPlanView.removeAll();
		for(NetworkElementType elementType : Constants.NetworkElementType.values())
		{
			if (routingType == RoutingType.SOURCE_ROUTING && elementType == NetworkElementType.FORWARDING_RULE) continue;
			if (routingType == RoutingType.HOP_BY_HOP_ROUTING && (elementType == NetworkElementType.PROTECTION_SEGMENT || elementType == NetworkElementType.ROUTE)) continue;
			netPlanView.addTab(netPlanViewTabName.get(elementType), netPlanViewTableComponent.get(elementType));
		}
		
		for(int tabId = 0; tabId < netPlanView.getTabCount(); tabId++)
		{
			if (netPlanView.getComponentAt(tabId).equals(selectedTab))
			{
				netPlanView.setSelectedIndex(tabId);
				break;
			}
		}
		
		networkAttributeTable.setEnabled(false);
		((DefaultTableModel) networkAttributeTable.getModel()).setDataVector(new Object[1][attributeTableHeader.length], attributeTableHeader);

		layerAttributeTable.setEnabled(false);
		((DefaultTableModel) layerAttributeTable.getModel()).setDataVector(new Object[1][attributeTableHeader.length], attributeTableHeader);

		EnumMap<NetworkElementType, Object[][]> networkElementInfo = new EnumMap<NetworkElementType, Object[][]>(NetworkElementType.class);

		NetPlan initialState = null;
		if (showInitialPlan != null && getInitialDesign().isLayerActive(layerId))
			initialState = getInitialDesign();

		RoutingType routingType_initialNetPlan = initialState == null ? null : initialState.getRoutingType();
		
		Map<String, String> net2planParameters = Configuration.getOptions();
		double PRECISION_FACTOR = Double.parseDouble(net2planParameters.get("precisionFactor"));

		Set<Long> nodeIds = currentState.getNodeIds();
		Set<Long> linkIds = currentState.getLinkIds();
		Set<Long> demandIds = currentState.getDemandIds();
		Set<Long> srgIds = currentState.getSRGIds();

		if (!nodeIds.isEmpty())
		{
			String[] nodeTableHeader = netPlanViewTableHeader.get(NetworkElementType.NODE);

			List<Object[]> allNodeData = new LinkedList<Object[]>();

			for(long nodeId : nodeIds)
			{
				Point2D nodeXYPosition = currentState.getNodeXYPosition(nodeId);
				Set<Long> srgIds_thisNode = currentState.getNodeSRGs(nodeId);
				double ingressTraffic = currentState.getNodeIngressTraffic(nodeId);
				double egressTraffic = currentState.getNodeEgressTraffic(nodeId);
				double traversingTraffic = currentState.getNodeTraversingTraffic(nodeId);

				Object[] nodeData = new Object[12];
				nodeData[0] = nodeId;
				nodeData[1] = topologyPanel.getCanvas().isNodeVisible(nodeId);
				nodeData[2] = currentState.getNodeName(nodeId);
				nodeData[3] = !currentState.isNodeDown(nodeId);
				nodeData[4] = nodeXYPosition.getX();
				nodeData[5] = nodeXYPosition.getY();
				nodeData[6] = ingressTraffic;
				nodeData[7] = egressTraffic;
				nodeData[8] = traversingTraffic;
				nodeData[9] = ingressTraffic + egressTraffic + traversingTraffic;
				nodeData[10] = srgIds_thisNode.isEmpty() ? "none" : CollectionUtils.join(srgIds_thisNode, ", ");
				nodeData[11] = StringUtils.mapToString(currentState.getNodeAttributeMap(nodeId));
				allNodeData.add(nodeData);

				if (showInitialPlan != null && initialState != null && initialState.isNodeActive(nodeId))
				{
					nodeXYPosition = initialState.getNodeXYPosition(nodeId);
					srgIds_thisNode = initialState.getNodeSRGs(nodeId);
					ingressTraffic = initialState.getNodeIngressTraffic(nodeId);
					egressTraffic = initialState.getNodeEgressTraffic(nodeId);
					traversingTraffic = initialState.getNodeTraversingTraffic(nodeId);

					Object[] nodeData_initialNetPlan = new Object[12];
					nodeData_initialNetPlan[0] = null;
					nodeData_initialNetPlan[1] = null;
					nodeData_initialNetPlan[2] = initialState.getNodeName(nodeId);
					nodeData_initialNetPlan[3] = initialState.isNodeDown(nodeId);
					nodeData_initialNetPlan[4] = nodeXYPosition.getX();
					nodeData_initialNetPlan[5] = nodeXYPosition.getY();
					nodeData_initialNetPlan[6] = ingressTraffic;
					nodeData_initialNetPlan[7] = egressTraffic;
					nodeData_initialNetPlan[8] = traversingTraffic;
					nodeData_initialNetPlan[9] = ingressTraffic + egressTraffic + traversingTraffic;
					nodeData_initialNetPlan[10] = srgIds_thisNode.isEmpty() ? "none" : CollectionUtils.join(srgIds_thisNode, ", ");
					nodeData_initialNetPlan[11] = StringUtils.mapToString(initialState.getNodeAttributeMap(nodeId));
					allNodeData.add(nodeData_initialNetPlan);
				}
			}

			networkElementInfo.put(NetworkElementType.NODE, allNodeData.toArray(new Object[allNodeData.size()][nodeTableHeader.length]));
		}

		Map<Long, Double> linkUtilizationMap = currentState.getLinkUtilizationMap();
		Map<Long, Double> linkUtilizationMap_initialNetPlan = initialState == null ? null : initialState.getLinkUtilizationMap();

		if (!linkIds.isEmpty())
		{
			String[] linkTableHeader = netPlanViewTableHeader.get(NetworkElementType.LINK);

			double max_rho_e = DoubleUtils.maxValue(linkUtilizationMap);
			double max_rho_e_initialNetPlan = initialState == null ? -1 : DoubleUtils.maxValue(linkUtilizationMap_initialNetPlan);

			List<Object[]> allLinkData = new LinkedList<Object[]>();
			for (long linkId : linkIds)
			{
				Set<Long> srgIds_thisLink = currentState.getLinkSRGs(linkId);
				Set<Long> traversingRoutes = routingType == RoutingType.SOURCE_ROUTING ? currentState.getLinkAssociatedRoutes(linkId) : new LinkedHashSet<Long>();
				Set<Long> traversingSegments = routingType == RoutingType.SOURCE_ROUTING ? currentState.getLinkAssociatedProtectionSegments(linkId) : new LinkedHashSet<Long>();
				Set<Pair<Long, Long>> forwardingRules = routingType == RoutingType.HOP_BY_HOP_ROUTING ? currentState.getLinkForwardingRules(linkId) : new LinkedHashSet<Pair<Long, Long>>();
				int numRoutes = traversingRoutes.size();
				int numSegments = traversingSegments.size();
				int numForwardingRules = forwardingRules.size();
				
				String routesString = numRoutes + (numRoutes > 0 ? " (" + CollectionUtils.join(traversingRoutes, ", ") + ")" : "");
				String segmentsString = numSegments + (numSegments > 0 ? " (" + CollectionUtils.join(traversingSegments, ", ") + ")" : "");
				StringBuilder forwardingRulesString = new StringBuilder(Integer.toString(numForwardingRules));
				if (numForwardingRules > 0)
				{
					forwardingRulesString.append(" (");
					Iterator<Pair<Long, Long>> it = forwardingRules.iterator();
					while(it.hasNext())
					{
						Pair<Long, Long> demandLinkPair = it.next();
						forwardingRulesString.append("d").append(demandLinkPair.getFirst()).append(" : ").append(String.format("%.3f", currentState.getForwardingRuleSplittingRatio(demandLinkPair)));
						if (it.hasNext()) forwardingRulesString.append(", ");
					}
					
					forwardingRulesString.append(")");
				}
				
				Pair<Long, Long> coupledDemand = currentState.isLinkCoupled(linkId) ? currentState.getLinkCoupledLowerLayerDemand(linkId) : null;
				
				Pair<Long, Long> nodePair_thisLink = currentState.getLinkOriginDestinationNodePair(linkId);
				long originNodeId = nodePair_thisLink.getFirst();
				long destinationNodeId = nodePair_thisLink.getSecond();
				String originNodeName = currentState.getNodeName(originNodeId);
				String destinationNodeName = currentState.getNodeName(destinationNodeId);

				double rho_e = currentState.getLinkUtilization(linkId);
				Object[] linkData = new Object[linkTableHeader.length];
				linkData[0] = linkId;
				linkData[1] = topologyPanel.getCanvas().isLinkVisible(linkId);
				linkData[2] = originNodeId + (originNodeName.isEmpty() ? "" : " (" + originNodeName + ")");
				linkData[3] = destinationNodeId + (destinationNodeName.isEmpty() ? "" : " (" + destinationNodeName + ")");
				linkData[4] = !currentState.isLinkDown(linkId);
				linkData[5] = coupledDemand == null ? "" : "d" + coupledDemand.getSecond() + " (layer " + coupledDemand.getFirst() + ")";
				linkData[6] = currentState.getLinkCapacity(linkId);
				linkData[7] = currentState.getLinkCarriedTraffic(linkId);
				linkData[8] = currentState.getLinkCapacityReservedForProtection(linkId);
				linkData[9] = rho_e;
				linkData[10] = currentState.getLinkUtilizationWithoutConsiderReservedCapacityForProtection(linkId);
				linkData[11] = DoubleUtils.isEqualWithinRelativeTolerance(max_rho_e, rho_e, PRECISION_FACTOR);
				linkData[12] = currentState.getLinkLengthInKm(linkId);
				linkData[13] = currentState.getLinkPropagationSpeedInKmPerSecond(linkId);
				linkData[14] = 1000 * currentState.getLinkPropagationDelayInSeconds(linkId);
				linkData[15] = routesString;
				linkData[16] = segmentsString;
				linkData[17] = forwardingRulesString.toString();
				linkData[18] = srgIds_thisLink.isEmpty() ? "none" : CollectionUtils.join(srgIds_thisLink, ", ");
				linkData[19] = StringUtils.mapToString(currentState.getLinkAttributeMap(linkId));
				allLinkData.add(linkData);

				if (showInitialPlan != null && initialState != null && initialState.isLinkActive(linkId))
				{
					srgIds_thisLink = initialState.getLinkSRGs(linkId);

					traversingRoutes = routingType_initialNetPlan == RoutingType.SOURCE_ROUTING ? initialState.getLinkAssociatedRoutes(linkId) : new LinkedHashSet<Long>();
					traversingSegments = routingType_initialNetPlan == RoutingType.SOURCE_ROUTING ? initialState.getLinkAssociatedProtectionSegments(linkId) : new LinkedHashSet<Long>();
					forwardingRules = routingType_initialNetPlan == RoutingType.HOP_BY_HOP_ROUTING ? initialState.getLinkForwardingRules(linkId) : new LinkedHashSet<Pair<Long, Long>>();
					numRoutes = traversingRoutes.size();
					numSegments = traversingSegments.size();
					numForwardingRules = forwardingRules.size();

					routesString = numRoutes + (numRoutes > 0 ? " (" + CollectionUtils.join(traversingRoutes, ", ") + ")" : "");
					segmentsString = numSegments + (numSegments > 0 ? " (" + CollectionUtils.join(traversingSegments, ", ") + ")" : "");
					forwardingRulesString = new StringBuilder(Integer.toString(numForwardingRules));
					if (numForwardingRules > 0)
					{
						forwardingRulesString.append(" (");
						Iterator<Pair<Long, Long>> it = forwardingRules.iterator();
						while(it.hasNext())
						{
							Pair<Long, Long> demandLinkPair = it.next();
							forwardingRulesString.append("d").append(demandLinkPair.getFirst()).append(" : ").append(String.format("%.3f", initialState.getForwardingRuleSplittingRatio(demandLinkPair)));
							if (it.hasNext()) forwardingRulesString.append(", ");
						}

						forwardingRulesString.append(")");
					}

					coupledDemand = initialState.isLinkCoupled(linkId) ? initialState.getLinkCoupledLowerLayerDemand(linkId) : null;
				
					double rho_e_initialNetPlan = initialState.getLinkUtilization(linkId);
					Object[] linkData_initialNetPlan = new Object[linkTableHeader.length];
					linkData_initialNetPlan[0] = null;
					linkData_initialNetPlan[1] = null;
					linkData_initialNetPlan[2] = null;
					linkData_initialNetPlan[3] = null;
					linkData_initialNetPlan[4] = initialState.isLinkDown(linkId);
					linkData_initialNetPlan[5] = coupledDemand == null ? "" : "d" + coupledDemand.getSecond() + " (layer " + coupledDemand.getFirst() + ")";
					linkData_initialNetPlan[6] = initialState.getLinkCapacity(linkId);
					linkData_initialNetPlan[7] = initialState.getLinkCarriedTraffic(linkId);
					linkData_initialNetPlan[8] = initialState.getLinkCapacityReservedForProtection(linkId);
					linkData_initialNetPlan[9] = rho_e_initialNetPlan;
					linkData_initialNetPlan[10] = initialState.getLinkUtilizationWithoutConsiderReservedCapacityForProtection(linkId);
					linkData_initialNetPlan[11] = DoubleUtils.isEqualWithinRelativeTolerance(max_rho_e_initialNetPlan, rho_e_initialNetPlan, PRECISION_FACTOR);
					linkData_initialNetPlan[12] = initialState.getLinkLengthInKm(linkId);
					linkData_initialNetPlan[13] = initialState.getLinkPropagationSpeedInKmPerSecond(linkId);
					linkData_initialNetPlan[14] = 1000 * initialState.getLinkPropagationDelayInSeconds(linkId);
					linkData_initialNetPlan[15] = routesString;
					linkData_initialNetPlan[16] = segmentsString;
					linkData_initialNetPlan[17] = forwardingRulesString.toString();
					linkData_initialNetPlan[18] = srgIds_thisLink.isEmpty() ? "none" : CollectionUtils.join(srgIds_thisLink, ", ");
					linkData_initialNetPlan[19] = StringUtils.mapToString(initialState.getLinkAttributeMap(linkId));
					allLinkData.add(linkData_initialNetPlan);
				}
			}

			networkElementInfo.put(NetworkElementType.LINK, allLinkData.toArray(new Object[allLinkData.size()][linkTableHeader.length]));
		}

		if (!demandIds.isEmpty())
		{
			String[] demandTableHeader = netPlanViewTableHeader.get(NetworkElementType.DEMAND);

			List<Object[]> allDemandData = new LinkedList<Object[]>();
			for(long demandId : demandIds)
			{
				Set<Long> routeIds_thisDemand = routingType == RoutingType.SOURCE_ROUTING ? currentState.getDemandRoutes(demandId) : new LinkedHashSet<Long>();

				Pair<Long, Long> coupledLink = currentState.isDemandCoupled(demandId) ? currentState.getDemandCoupledUpperLayerLink(demandId) : null;
				
				Pair<Long, Long> nodePair_thisDemand = currentState.getDemandIngressEgressNodePair(demandId);
				long ingressNodeId = nodePair_thisDemand.getFirst();
				long egressNodeId = nodePair_thisDemand.getSecond();
				String ingressNodeName = currentState.getNodeName(ingressNodeId);
				String egressNodeName = currentState.getNodeName(egressNodeId);

				double h_d = currentState.getDemandOfferedTraffic(demandId);
				double lostTraffic_d = currentState.getDemandBlockedTraffic(demandId);
				Object[] demandData = new Object[demandTableHeader.length];
				demandData[0] = demandId;
				demandData[1] = ingressNodeId + (ingressNodeName.isEmpty() ? "" : " (" + ingressNodeName + ")");
				demandData[2] = egressNodeId + (egressNodeName.isEmpty() ? "" : " (" + egressNodeName + ")");
				demandData[3] = coupledLink == null ? "" : "e" + coupledLink.getSecond() + " (layer " + coupledLink.getFirst() + ")";
				demandData[4] = h_d;
				demandData[5] = currentState.getDemandCarriedTraffic(demandId);
				demandData[6] = h_d == 0 ? 0 : 100 * lostTraffic_d / h_d;
				demandData[7] = currentState.getDemandRoutingCycleType(demandId);
				demandData[8] = routingType == RoutingType.HOP_BY_HOP_ROUTING ? "-" : (currentState.isDemandBifurcated(demandId) ? String.format("Yes (%d)", currentState.getDemandBifurcationDegree(demandId)) : "No");
				demandData[9] = routeIds_thisDemand.size() + (routeIds_thisDemand.size() > 0 ? " (" + CollectionUtils.join(routeIds_thisDemand, ",") + ")" : "");
				demandData[10] = StringUtils.mapToString(currentState.getDemandAttributeMap(demandId));
				allDemandData.add(demandData);

				if (showInitialPlan != null && initialState != null && initialState.isDemandActive(demandId))
				{
					Set<Long> routeIds_thisDemand_initialNetPlan = routingType_initialNetPlan == RoutingType.SOURCE_ROUTING ? initialState.getDemandRoutes(demandId) : new LinkedHashSet<Long>();

					double h_d_initialNetPlan = initialState.getDemandOfferedTraffic(demandId);
					double lostTraffic_d_initialNetPlan = initialState.getDemandBlockedTraffic(demandId);
					Object[] demandData_initialNetPlan = new Object[demandTableHeader.length];
					demandData_initialNetPlan[0] = null;
					demandData_initialNetPlan[1] = null;
					demandData_initialNetPlan[2] = null;
					demandData_initialNetPlan[3] = coupledLink == null ? "" : "e" + coupledLink.getSecond() + " (layer " + coupledLink.getFirst() + ")";
					demandData_initialNetPlan[4] = h_d_initialNetPlan;
					demandData_initialNetPlan[5] = initialState.getDemandCarriedTraffic(demandId);
					demandData_initialNetPlan[6] = h_d_initialNetPlan == 0 ? 0 : 100 * lostTraffic_d_initialNetPlan / h_d_initialNetPlan;
					demandData_initialNetPlan[7] = initialState.getDemandRoutingCycleType(demandId);
					demandData_initialNetPlan[8] = routingType_initialNetPlan == RoutingType.HOP_BY_HOP_ROUTING ? "-" : (initialState.isDemandBifurcated(demandId) ? String.format("Yes (%d)", initialState.getDemandBifurcationDegree(demandId)) : "No");
					demandData_initialNetPlan[9] = routeIds_thisDemand_initialNetPlan.size() + (routeIds_thisDemand_initialNetPlan.size() > 0 ? " (" + CollectionUtils.join(routeIds_thisDemand_initialNetPlan, ",") + ")" : "");
					demandData_initialNetPlan[10] = StringUtils.mapToString(initialState.getDemandAttributeMap(demandId));
					allDemandData.add(demandData_initialNetPlan);
				}
			}

			networkElementInfo.put(NetworkElementType.DEMAND, allDemandData.toArray(new Object[allDemandData.size()][demandTableHeader.length]));
		}
		
		boolean sameRoutingType = initialState != null && initialState.getRoutingType() == routingType;

		switch(routingType)
		{
			case HOP_BY_HOP_ROUTING:
			{
				hopByHopRoutingActivated.setSelected(true);
				
				Set<Pair<Long, Long>> demandLinkPairs = currentState.getForwardingRules();
				if (!demandLinkPairs.isEmpty())
				{
					String[] forwardingRuleTableHeader = netPlanViewTableHeader.get(NetworkElementType.FORWARDING_RULE);

					List<Object[]> allForwardingRuleData = new LinkedList<Object[]>();
					for(Pair<Long, Long> demandLinkPair : demandLinkPairs)
					{
						long demandId = demandLinkPair.getFirst();
						Pair<Long, Long> nodePair_thisDemand = currentState.getDemandIngressEgressNodePair(demandId);
						long ingressNodeId = nodePair_thisDemand.getFirst();
						long egressNodeId = nodePair_thisDemand.getSecond();
						String ingressNodeName = currentState.getNodeName(ingressNodeId);
						String egressNodeName = currentState.getNodeName(egressNodeId);

						long linkId = demandLinkPair.getSecond();
						Pair<Long, Long> nodePair_thisLink = currentState.getLinkOriginDestinationNodePair(linkId);
						long originNodeId = nodePair_thisLink.getFirst();
						long destinationNodeId = nodePair_thisLink.getSecond();
						String originNodeName = currentState.getNodeName(nodePair_thisLink.getFirst());
						String destinationNodeName = currentState.getNodeName(nodePair_thisLink.getSecond());

						Object[] forwardingRuleData = new Object[forwardingRuleTableHeader.length];
						forwardingRuleData[0] = originNodeId + (originNodeName.isEmpty() ? "" : " (" + originNodeName + ")");
						forwardingRuleData[1] = demandId + " [" + ingressNodeId + (ingressNodeName.isEmpty() ? "" : " (" + ingressNodeName + ")") + " -> " + egressNodeId + (egressNodeName.isEmpty() ? "" : " (" + egressNodeName + ")") + "]";
						forwardingRuleData[2] = linkId + " [" + originNodeId + (originNodeName.isEmpty() ? "" : " (" + originNodeName + ")") + " -> " + destinationNodeId + (destinationNodeName.isEmpty() ? "" : " (" + destinationNodeName + ")") + "]";
						forwardingRuleData[3] = currentState.getForwardingRuleSplittingRatio(demandLinkPair);
						forwardingRuleData[4] = currentState.getForwardingRuleCarriedTraffic(demandLinkPair);
						forwardingRuleData[5] = StringUtils.mapToString(currentState.getForwardingRuleAttributeMap(demandLinkPair));
						allForwardingRuleData.add(forwardingRuleData);

						if (showInitialPlan != null && initialState != null && sameRoutingType && initialState.isForwardingRuleActive(demandLinkPair))
						{
							Object[] forwardingRuleData_initialNetPlan = new Object[forwardingRuleTableHeader.length];
							forwardingRuleData_initialNetPlan[0] = null;
							forwardingRuleData_initialNetPlan[1] = null;
							forwardingRuleData_initialNetPlan[2] = null;
							forwardingRuleData_initialNetPlan[3] = initialState.getForwardingRuleSplittingRatio(demandLinkPair);
							forwardingRuleData_initialNetPlan[4] = initialState.getForwardingRuleCarriedTraffic(demandLinkPair);
							forwardingRuleData_initialNetPlan[5] = StringUtils.mapToString(initialState.getForwardingRuleAttributeMap(demandLinkPair));
							allForwardingRuleData.add(forwardingRuleData_initialNetPlan);
						}
					}

					networkElementInfo.put(NetworkElementType.FORWARDING_RULE, allForwardingRuleData.toArray(new Object[allForwardingRuleData.size()][forwardingRuleTableHeader.length]));

					JTable forwardingRuleTable = netPlanViewTable.get(NetworkElementType.FORWARDING_RULE);
					Object[][] forwardingRuleTableData = networkElementInfo.get(NetworkElementType.FORWARDING_RULE);
					forwardingRuleTable.setEnabled(true);
					((DefaultTableModel) forwardingRuleTable.getModel()).setDataVector(forwardingRuleTableData, forwardingRuleTableHeader);
				}

				break;
			}	
			
			case SOURCE_ROUTING:
			{
				sourceRoutingActivated.setSelected(true);
				
				Set<Long> routeIds = currentState.getRouteIds();
				if (!routeIds.isEmpty())
				{
					String[] routeTableHeader = netPlanViewTableHeader.get(NetworkElementType.ROUTE);

					List<Object[]> allRouteData = new LinkedList<Object[]>();
					for(long routeId : routeIds)
					{
						long demandId = currentState.getRouteDemand(routeId);

						List<Long> seqLinks = currentState.getRouteSequenceOfLinks(routeId);
						List<Long> seqNodes = currentState.getRouteSequenceOfNodes(routeId);
						Map<Long, Double> utilizationMap = CollectionUtils.selectEntries(linkUtilizationMap, new LinkedHashSet<Long>(seqLinks));
						double maxUtilization = utilizationMap.isEmpty() ? 0 : DoubleUtils.maxValue(utilizationMap);

						Pair<Long, Long> nodePair_thisDemand = currentState.getDemandIngressEgressNodePair(demandId);
						long ingressNodeId = nodePair_thisDemand.getFirst();
						long egressNodeId = nodePair_thisDemand.getSecond();
						String ingressNodeName = currentState.getNodeName(ingressNodeId);
						String egressNodeName = currentState.getNodeName(egressNodeId);

						Object[] routeData = new Object[routeTableHeader.length];
						routeData[0] = routeId;
						routeData[1] = demandId;
						routeData[2] = ingressNodeId + (ingressNodeName.isEmpty() ? "" : " (" + ingressNodeName + ")");
						routeData[3] = egressNodeId + (egressNodeName.isEmpty() ? "" : " (" + egressNodeName + ")");
						routeData[4] = currentState.getDemandOfferedTraffic(demandId);
						routeData[5] = currentState.getRouteCarriedTraffic(routeId);
						routeData[6] = currentState.getRouteOccupiedCapacity(routeId);
						routeData[7] = CollectionUtils.join(seqLinks, " => ");
						routeData[8] = CollectionUtils.join(seqNodes, " => ");
						routeData[9] = currentState.getRouteNumberOfHops(routeId);
						routeData[10] = currentState.getRouteLengthInKm(routeId);
						routeData[11] = 1000 * currentState.getRoutePropagationDelayInSeconds(routeId);
						routeData[12] = maxUtilization;
						routeData[13] = CollectionUtils.join(currentState.getRouteBackupSegmentList(routeId), ", ");
						routeData[14] = StringUtils.mapToString(currentState.getRouteAttributeMap(routeId));
						allRouteData.add(routeData);

						if (showInitialPlan != null && initialState != null && sameRoutingType && initialState.isRouteActive(routeId))
						{
							seqLinks = initialState.getRouteSequenceOfLinks(routeId);
							seqNodes = initialState.getRouteSequenceOfNodes(routeId);
							utilizationMap = CollectionUtils.selectEntries(linkUtilizationMap_initialNetPlan, new LinkedHashSet<Long>(seqLinks));
							maxUtilization = utilizationMap.isEmpty() ? 0 : DoubleUtils.maxValue(utilizationMap);

							Object[] routeData_initialNetPlan = new Object[routeTableHeader.length];
							routeData_initialNetPlan[0] = null;
							routeData_initialNetPlan[1] = null;
							routeData_initialNetPlan[2] = null;
							routeData_initialNetPlan[3] = null;
							routeData_initialNetPlan[4] = initialState.getDemandOfferedTraffic(demandId);
							routeData_initialNetPlan[5] = initialState.getRouteCarriedTraffic(routeId);
							routeData_initialNetPlan[6] = initialState.getRouteOccupiedCapacity(routeId);
							routeData_initialNetPlan[7] = CollectionUtils.join(seqLinks, " => ");
							routeData_initialNetPlan[8] = CollectionUtils.join(seqNodes, " => ");
							routeData_initialNetPlan[9] = initialState.getRouteNumberOfHops(routeId);
							routeData_initialNetPlan[10] = initialState.getRouteLengthInKm(routeId);
							routeData_initialNetPlan[11] = 1000 * initialState.getRoutePropagationDelayInSeconds(routeId);
							routeData_initialNetPlan[12] = maxUtilization;
							routeData_initialNetPlan[13] = CollectionUtils.join(initialState.getRouteBackupSegmentList(routeId), ", ");
							routeData_initialNetPlan[14] = StringUtils.mapToString(initialState.getRouteAttributeMap(routeId));
							allRouteData.add(routeData_initialNetPlan);
						}
					}

					networkElementInfo.put(NetworkElementType.ROUTE, allRouteData.toArray(new Object[allRouteData.size()][routeTableHeader.length]));

					JTable routeTable = netPlanViewTable.get(NetworkElementType.ROUTE);
					Object[][] routeTableData = networkElementInfo.get(NetworkElementType.ROUTE);
					routeTable.setEnabled(true);
					((DefaultTableModel) routeTable.getModel()).setDataVector(routeTableData, routeTableHeader);
				}
				
				Set<Long> segmentIds = currentState.getProtectionSegmentIds();
				if (!segmentIds.isEmpty())
				{
					String[] segmentTableHeader = netPlanViewTableHeader.get(NetworkElementType.PROTECTION_SEGMENT);

					List<Object[]> allSegmentData = new LinkedList<Object[]>();
					for(long segmentId : segmentIds)
					{
						List<Long> seqLinks = currentState.getProtectionSegmentSequenceOfLinks(segmentId);
						List<Long> seqNodes = currentState.getProtectionSegmentSequenceOfNodes(segmentId);
						Set<Long> routeIds_thisSegment = currentState.getProtectionSegmentRoutes(segmentId);
						int numRoutes = routeIds_thisSegment.size();
						
						Pair<Long, Long> nodePair_thisSegment = currentState.getProtectionSegmentOriginDestinationNodePair(segmentId);
						long originNodeId = nodePair_thisSegment.getFirst();
						long destinationNodeId = nodePair_thisSegment.getSecond();
						String originNodeName = currentState.getNodeName(originNodeId);
						String destinationNodeName = currentState.getNodeName(destinationNodeId);

						Object[] segmentData = new Object[segmentTableHeader.length];
						segmentData[0] = segmentId;
						segmentData[1] = originNodeId + (originNodeName.isEmpty() ? "" : " (" + originNodeName + ")");
						segmentData[2] = destinationNodeId + (destinationNodeName.isEmpty() ? "" : " (" + destinationNodeName + ")");
						segmentData[3] = currentState.getProtectionSegmentReservedCapacity(segmentId);
						segmentData[4] = currentState.getProtectionSegmentCarriedTraffic(segmentId);
						segmentData[5] = CollectionUtils.join(seqLinks, " => ");
						segmentData[6] = CollectionUtils.join(seqNodes, " => ");
						segmentData[7] = currentState.getProtectionSegmentNumberOfHops(segmentId);
						segmentData[8] = currentState.getProtectionSegmentLengthInKm(segmentId);
						segmentData[9] = 1000 * currentState.getProtectionSegmentPropagationDelayInSeconds(segmentId);
						segmentData[10] = numRoutes > 1 ? "Shared" : (numRoutes == 0 ? "Not used" : "Dedicated");
						segmentData[11] = numRoutes + (numRoutes > 0 ? " (" + CollectionUtils.join(routeIds_thisSegment, ", ") + ")" : "");
						segmentData[12] = StringUtils.mapToString(currentState.getProtectionSegmentAttributeMap(segmentId));
						allSegmentData.add(segmentData);

						if (showInitialPlan != null && initialState != null && sameRoutingType && initialState.isProtectionSegmentActive(segmentId))
						{
							routeIds_thisSegment = initialState.getProtectionSegmentRoutes(segmentId);
							numRoutes = routeIds_thisSegment.size();

							Object[] segmentData_initialNetPlan = new Object[segmentTableHeader.length];
							segmentData_initialNetPlan[0] = null;
							segmentData_initialNetPlan[1] = null;
							segmentData_initialNetPlan[2] = null;
							segmentData_initialNetPlan[3] = initialState.getProtectionSegmentReservedCapacity(segmentId);
							segmentData_initialNetPlan[4] = initialState.getProtectionSegmentCarriedTraffic(segmentId);
							segmentData_initialNetPlan[5] = null;
							segmentData_initialNetPlan[6] = null;
							segmentData_initialNetPlan[7] = initialState.getProtectionSegmentNumberOfHops(segmentId);
							segmentData_initialNetPlan[8] = initialState.getProtectionSegmentLengthInKm(segmentId);
							segmentData_initialNetPlan[9] = 1000 * initialState.getProtectionSegmentPropagationDelayInSeconds(segmentId);
							segmentData_initialNetPlan[10] = numRoutes > 1 ? "Shared" : (numRoutes == 0 ? "Not used" : "Dedicated");
							segmentData_initialNetPlan[11] = numRoutes + (numRoutes > 0 ? " (" + CollectionUtils.join(routeIds_thisSegment, ", ") + ")" : "");
							segmentData_initialNetPlan[12] = StringUtils.mapToString(initialState.getProtectionSegmentAttributeMap(segmentId));
							allSegmentData.add(segmentData_initialNetPlan);
						}

					}

					networkElementInfo.put(NetworkElementType.PROTECTION_SEGMENT, allSegmentData.toArray(new Object[allSegmentData.size()][segmentTableHeader.length]));

					JTable segmentTable = netPlanViewTable.get(NetworkElementType.PROTECTION_SEGMENT);
					Object[][] segmentTableData = networkElementInfo.get(NetworkElementType.PROTECTION_SEGMENT);
					segmentTable.setEnabled(true);
					((DefaultTableModel) segmentTable.getModel()).setDataVector(segmentTableData, segmentTableHeader);
				}
				
				break;
			}
			
			default:
				throw new RuntimeException("Bad");
		}

		if (!srgIds.isEmpty())
		{
			String[] srgTableHeader = netPlanViewTableHeader.get(NetworkElementType.SRG);

			List<Object[]> allSRGData = new LinkedList<Object[]>();
			for(long srgId : srgIds)
			{
				double mttfInHours = currentState.getSRGMeanTimeToFailInHours(srgId);
				double mttrInHours = currentState.getSRGMeanTimeToRepairInHours(srgId);
				Set<Long> routeIds_thisSRG = routingType == RoutingType.SOURCE_ROUTING ? currentState.getSRGRoutes(layerId, srgId) : new LinkedHashSet<Long>();
				Set<Long> segmentIds_thisSRG = routingType == RoutingType.SOURCE_ROUTING ? currentState.getSRGProtectionSegments(layerId, srgId) : new LinkedHashSet<Long>();
				int numRoutes = routeIds_thisSRG.size();
				int numSegments = segmentIds_thisSRG.size();

				Object[] srgData = new Object[srgTableHeader.length];
				srgData[0] = srgId;
				srgData[1] = mttfInHours;
				srgData[2] = mttrInHours;
				srgData[3] = CollectionUtils.join(currentState.getSRGNodes(srgId), ", ");
				srgData[4] = CollectionUtils.join(currentState.getSRGLinks(layerId, srgId), ", ");
				srgData[5] = numRoutes + (numRoutes > 0 ? " (" + CollectionUtils.join(routeIds_thisSRG, ", ") + ")" : "");
				srgData[6] = numSegments + (numSegments > 0 ? " (" + CollectionUtils.join(segmentIds_thisSRG, ", ") + ")" : "");
				srgData[7] = StringUtils.mapToString(currentState.getSRGAttributeMap(srgId));
				allSRGData.add(srgData);

				if (showInitialPlan != null && initialState != null && initialState.isSRGActive(srgId))
				{
					mttfInHours = initialState.getSRGMeanTimeToFailInHours(srgId);
					mttrInHours = initialState.getSRGMeanTimeToRepairInHours(srgId);
					routeIds_thisSRG = routingType_initialNetPlan == RoutingType.SOURCE_ROUTING ? initialState.getSRGRoutes(layerId, srgId) : new LinkedHashSet<Long>();
					segmentIds_thisSRG = routingType_initialNetPlan == RoutingType.SOURCE_ROUTING ? initialState.getSRGProtectionSegments(layerId, srgId) : new LinkedHashSet<Long>();
					numRoutes = routeIds_thisSRG.size();
					numSegments = segmentIds_thisSRG.size();

					Object[] srgData_initialNetPlan = new Object[srgTableHeader.length];
					srgData_initialNetPlan[0] = null;
					srgData_initialNetPlan[1] = mttfInHours;
					srgData_initialNetPlan[2] = mttrInHours;
					srgData_initialNetPlan[3] = CollectionUtils.join(initialState.getSRGNodes(srgId), ", ");
					srgData_initialNetPlan[4] = CollectionUtils.join(initialState.getSRGLinks(layerId, srgId), ", ");
					srgData_initialNetPlan[5] = numRoutes + (numRoutes > 0 ? " (" + CollectionUtils.join(routeIds_thisSRG, ", ") + ")" : "");
					srgData_initialNetPlan[6] = numSegments + (numSegments > 0 ? " (" + CollectionUtils.join(segmentIds_thisSRG, ", ") + ")" : "");
					srgData_initialNetPlan[7] = StringUtils.mapToString(initialState.getSRGAttributeMap(srgId));
					allSRGData.add(srgData_initialNetPlan);
				}
			}

			networkElementInfo.put(NetworkElementType.SRG, allSRGData.toArray(new Object[allSRGData.size()][srgTableHeader.length]));
		}
		
		initialState = getInitialDesign();
		Set<Long> layerIds = currentState.getLayerIds();
		String[] layerTableHeader = netPlanViewTableHeader.get(NetworkElementType.LAYER);
		List<Object[]> allLayerData = new LinkedList<Object[]>();
		for(long auxLayerId : layerIds)
		{
			RoutingType routingType_thisLayer = currentState.getRoutingType(auxLayerId);
			Object[] layerData = new Object[layerTableHeader.length];
			layerData[0] = auxLayerId;
			layerData[1] = currentState.getLayerName(auxLayerId);
			layerData[2] = currentState.getRoutingType(auxLayerId);
			layerData[3] = currentState.getNumberOfLinks(auxLayerId);
			layerData[4] = currentState.getNumberOfDemands(auxLayerId);
			layerData[5] = routingType_thisLayer == RoutingType.SOURCE_ROUTING ? currentState.getNumberOfRoutes(auxLayerId) : 0;
			layerData[6] = routingType_thisLayer == RoutingType.HOP_BY_HOP_ROUTING ? currentState.getNumberOfForwardingRules(auxLayerId) : 0;
			layerData[7] = routingType_thisLayer == RoutingType.SOURCE_ROUTING ? currentState.getNumberOfProtectionSegments(auxLayerId) : 0;
			layerData[8] = currentState.getLayerDescription(auxLayerId);
			layerData[9] = currentState.getLinkCapacityUnitsName(auxLayerId);
			layerData[10] = currentState.getDemandTrafficUnitsName(auxLayerId);
			layerData[11] = StringUtils.mapToString(currentState.getLayerAttributeMap(auxLayerId));
			allLayerData.add(layerData);

			if (showInitialPlan != null && initialState != null && initialState.isLayerActive(auxLayerId))
			{
				RoutingType routingType_thisLayer_initialNetPlan = initialState.getRoutingType(auxLayerId);
				Object[] layerData_initialNetPlan = new Object[layerTableHeader.length];
				layerData_initialNetPlan[0] = null;
				layerData_initialNetPlan[1] = initialState.getLayerName(auxLayerId);
				layerData_initialNetPlan[2] = initialState.getRoutingType(auxLayerId);
				layerData_initialNetPlan[3] = initialState.getNumberOfLinks(auxLayerId);
				layerData_initialNetPlan[4] = initialState.getNumberOfDemands(auxLayerId);
				layerData_initialNetPlan[5] = routingType_thisLayer_initialNetPlan == RoutingType.SOURCE_ROUTING ? initialState.getNumberOfRoutes(auxLayerId) : 0;
				layerData_initialNetPlan[6] = routingType_thisLayer_initialNetPlan == RoutingType.HOP_BY_HOP_ROUTING ? initialState.getNumberOfForwardingRules(auxLayerId) : 0;
				layerData_initialNetPlan[7] = routingType_thisLayer_initialNetPlan == RoutingType.SOURCE_ROUTING ? initialState.getNumberOfProtectionSegments(auxLayerId) : 0;
				layerData_initialNetPlan[8] = initialState.getLayerDescription(auxLayerId);
				layerData_initialNetPlan[9] = initialState.getLinkCapacityUnitsName(auxLayerId);
				layerData_initialNetPlan[10] = initialState.getDemandTrafficUnitsName(auxLayerId);
				layerData_initialNetPlan[11] = StringUtils.mapToString(initialState.getLayerAttributeMap(auxLayerId));
				allLayerData.add(layerData_initialNetPlan);
			}

			networkElementInfo.put(NetworkElementType.LAYER, allLayerData.toArray(new Object[allLayerData.size()][layerTableHeader.length]));
		}
		
		if (currentState.hasNodes())
		{
			JTable nodeTable = netPlanViewTable.get(NetworkElementType.NODE);
			String[] nodeTableHeader = netPlanViewTableHeader.get(NetworkElementType.NODE);
			Object[][] nodeTableData = networkElementInfo.get(NetworkElementType.NODE);
			nodeTable.setEnabled(true);
			((DefaultTableModel) nodeTable.getModel()).setDataVector(nodeTableData, nodeTableHeader);
		}

		if (currentState.hasLinks())
		{
			JTable linkTable = netPlanViewTable.get(NetworkElementType.LINK);
			String[] linkTableHeader = netPlanViewTableHeader.get(NetworkElementType.LINK);
			Object[][] linkTableData = networkElementInfo.get(NetworkElementType.LINK);
			linkTable.setEnabled(true);
			((DefaultTableModel) linkTable.getModel()).setDataVector(linkTableData, linkTableHeader);
		}

		if (currentState.hasDemands())
		{
			JTable demandTable = netPlanViewTable.get(NetworkElementType.DEMAND);
			String[] demandTableHeader = netPlanViewTableHeader.get(NetworkElementType.DEMAND);
			Object[][] demandTableData = networkElementInfo.get(NetworkElementType.DEMAND);
			demandTable.setEnabled(true);
			((DefaultTableModel) demandTable.getModel()).setDataVector(demandTableData, demandTableHeader);
		}

		if (currentState.hasSRGs())
		{
			JTable srgTable = netPlanViewTable.get(NetworkElementType.SRG);
			String[] srgTableHeader = netPlanViewTableHeader.get(NetworkElementType.SRG);
			Object[][] srgTableData = networkElementInfo.get(NetworkElementType.SRG);
			srgTable.setEnabled(true);
			((DefaultTableModel) srgTable.getModel()).setDataVector(srgTableData, srgTableHeader);
		}

		txt_numLayers.setText(Integer.toString(currentState.getNumberOfLayers()));
		txt_numNodes.setText(Integer.toString(currentState.getNumberOfNodes()));
		txt_numSRGs.setText(Integer.toString(currentState.getNumberOfSRGs()));

		JTable layerTable = netPlanViewTable.get(NetworkElementType.LAYER);
		Object[][] layerTableData = networkElementInfo.get(NetworkElementType.LAYER);
		layerTable.setEnabled(true);
		((DefaultTableModel) layerTable.getModel()).setDataVector(layerTableData, layerTableHeader);

		Map<String, String> layerAttributes = currentState.getLayerAttributeMap(layerId);
		if (!layerAttributes.isEmpty())
		{
			int layerAttributeId = 0;
			Object[][] layerData = new Object[layerAttributes.size()][2];
			for (Map.Entry<String, String> entry : layerAttributes.entrySet())
			{
				layerData[layerAttributeId][0] = entry.getKey();
				layerData[layerAttributeId][1] = entry.getValue();
				layerAttributeId++;
			}

			((DefaultTableModel) layerAttributeTable.getModel()).setDataVector(layerData, attributeTableHeader);
		}

		Map<String, String> networkAttributes = currentState.getNetworkAttributeMap();
		if (!networkAttributes.isEmpty())
		{
			int networkAttributeId = 0;
			Object[][] networkData = new Object[networkAttributes.size()][2];
			for (Map.Entry<String, String> entry : networkAttributes.entrySet())
			{
				networkData[networkAttributeId][0] = entry.getKey();
				networkData[networkAttributeId][1] = entry.getValue();
				networkAttributeId++;
			}

			((DefaultTableModel) networkAttributeTable.getModel()).setDataVector(networkData, attributeTableHeader);
		}

		boolean isEditable_old = isEditable;
		isEditable = false;

		txt_networkName.setText(currentState.getNetworkName());
		txt_networkDescription.setText(currentState.getNetworkDescription());
		txt_networkDescription.setCaretPosition(0);
		txt_layerName.setText(currentState.getLayerName(layerId));
		txt_layerDescription.setText(currentState.getLayerDescription(layerId));
		txt_layerDescription.setCaretPosition(0);
		txt_layerLinkCapacityUnits.setText(currentState.getLinkCapacityUnitsName());
		txt_layerDemandTrafficUnits.setText(currentState.getDemandTrafficUnitsName());

		isEditable = isEditable_old;

		boolean hardComputations = currentState.getNumberOfNodes() <= 100;
		updateLayerMetrics(currentState, hardComputations);
	}
	
	@Override
	public void updateWarnings()
	{
		Map<String, String> net2planParameters = Configuration.getOptions();
		List<String> warnings = NetworkPerformanceMetrics.checkNetworkState(getDesign(), net2planParameters);
		String warningMsg = warnings.isEmpty() ? "Design is successfully completed!" : StringUtils.join(warnings, StringUtils.getLineSeparator());
		updateLog(warningMsg);
	}
	
	/**
	 * Adds a new tab in the right panel at the last position.
	 * 
	 * @param name Tab name
	 * @param tab Tab component
	 * @return Tab position
	 * @since 0.3.0
	 */
	protected final int addTab(String name, JComponent tab)
	{
		return addTab(name, tab, -1);
	}
	
	/**
	 * Adds a new tab in the right panel at the given position.
	 * 
	 * @param name Tab name
	 * @param tab Tab component
	 * @param tabIndex Tab position (-1 means last position)
	 * @return Tab position
	 * @since 0.3.0
	 */
	protected final int addTab(String name, JComponent tab, int tabIndex)
	{
		int numTabs = rightPane.getTabCount();
		if (numTabs == 9) throw new RuntimeException("A maximum of 9 tabs are allowed");
		
		if (tabIndex == -1) tabIndex = numTabs;
		rightPane.insertTab(name, null, tab, null, tabIndex);
		
		if (tabIndex <= viewNetPlanTabIndex) viewNetPlanTabIndex++;
		return tabIndex;
	}
	
	/**
	 * Indicates whether or not users are allowed to modify the 
	 * <code>NetPlan</code> object manually. 
	 *
	 * @return <code>true</code> if users can edit manually the <code>NetPlan</code> object. Otherwise, <code>false</code>.
	 * @since 0.3.0
	 */
	protected boolean allowEditNetPlanManually()
	{
		return false;
	}
	
	/**
	 * Indicates whether or not traffic demands can be added to the current design 
	 * from an external file.
	 *
	 * @return <code>true</code> if it is allowed to load traffic demands. Otherwise, <code>false</code>.
	 * @since 0.3.0
	 */
	protected boolean allowLoadTrafficDemands()
	{
		return false;
	}
	
	/**
	 * Indicates whether or not the initial <code>NetPlan</code> object is stored to be 
	 * compared with the current one (i.e. after some simulation steps).
	 *
	 * @return <code>true</code> if the initial <code>NetPlan</code> object is stored. Otherwise, <code>false</code>.
	 * @since 0.3.0
	 */
	protected boolean allowShowInitialNetPlan()
	{
		return false;
	}
	
	/**
	 * Asks user to confirm plugin reset.
	 * 
	 * @return <code>true</code> if user confirms to reset the plugin, or <code>false</code> otherwise
	 * @since 0.2.3
	 */
	protected static boolean askForReset()
	{
		int result = JOptionPane.showConfirmDialog(null, "Are you sure you want to reset? This will remove all unsaved data", "Reset", JOptionPane.YES_NO_OPTION);

		return result == JOptionPane.YES_OPTION;
	}

	private void configureLayerChooser()
	{
		layerChooser = new WiderJComboBox();
		layerChooserPane = new JPanel(new BorderLayout());
		layerChooserPane.add(new JLabel("Select layer: "), BorderLayout.WEST);
		layerChooserPane.add(layerChooser, BorderLayout.CENTER);
		layerChooser.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				Object selectedItem = layerChooser.getSelectedItem();
				if (!(selectedItem instanceof StringLabeller)) ErrorHandling.showErrorDialog("Bad object", "Error selecting layer");
				
				long layerId = (Long) ((StringLabeller) selectedItem).getObject();
				NetPlan currentState = getDesign();
				currentState.setLayerDefaultId(layerId);

				NetPlan initialState = getInitialDesign();
				if (initialState != null && initialState.isLayerActive(layerId)) initialState.setLayerDefaultId(layerId);

				RoutingType routingType = currentState.getRoutingType();
				routingSchemes.clearSelection();
				switch(routingType)
				{
					case HOP_BY_HOP_ROUTING:
						hopByHopRoutingActivated.setSelected(true);
						break;

					case SOURCE_ROUTING:
						sourceRoutingActivated.setSelected(true);
						break;

					default:
						throw new RuntimeException("Bad");
				}

				topologyPanel.getCanvas().updateTopology(currentState);
				updateNetPlanView();

				layerChanged(layerId);
			}
		});
	}
	
	/**
	 * Allows to include a custom panel in the left-bottom corner of the window, 
	 * just below the topologyPanel panel.
	 * 
	 * @return A panel to be included in the left-bottom corner of the window
	 * @since 0.3.0
	 */
	protected JPanel configureLeftBottomPanel()
	{
		return null;
	}
	
	private void configureNetPlanView()
	{
		isEditable = allowEditNetPlanManually();
		
		netPlanViewTable = new EnumMap<NetworkElementType, JTable>(NetworkElementType.class);
		netPlanViewTableDecorator = new EnumMap<NetworkElementType, FixedColumnDecorator>(NetworkElementType.class);
		netPlanViewTableComponent = new EnumMap<NetworkElementType, JComponent>(NetworkElementType.class);
		
		txt_layerName = new JTextField();
		txt_layerDescription = new JTextArea();
		txt_layerDescription.setFont(new JLabel().getFont());
		txt_layerDescription.setLineWrap(true);
		txt_layerDescription.setWrapStyleWord(true);
		txt_layerDemandTrafficUnits = new JTextField();
		txt_layerLinkCapacityUnits = new JTextField();
		txt_numLayers = new JTextField(); txt_numLayers.setEditable(false);
		txt_numNodes = new JTextField(); txt_numNodes.setEditable(false);
		txt_numSRGs = new JTextField(); txt_numSRGs.setEditable(false);
		
		sourceRoutingActivated = new JRadioButton("Source routing", false);
		sourceRoutingActivated.setEnabled(isEditable);
		hopByHopRoutingActivated = new JRadioButton("Hop-by-hop routing", false);
		hopByHopRoutingActivated.setEnabled(isEditable);
		
		if (isEditable)
		{
			ItemListener itemRoutingTypeListener = new ItemListener()
			{
				@Override
				public void itemStateChanged(ItemEvent event)
				{
					JRadioButton button = (JRadioButton) event.getSource();
					int state = event.getStateChange();

					NetPlan netPlan = getDesign();
					RoutingType previousRoutingType = netPlan.getRoutingType();
					
					if (button == sourceRoutingActivated && state == ItemEvent.SELECTED)
					{
						netPlan.setRoutingType(RoutingType.SOURCE_ROUTING);
						if (previousRoutingType != RoutingType.SOURCE_ROUTING)
							updateNetPlanView();
					}
						
					if (button == hopByHopRoutingActivated && state == ItemEvent.SELECTED)
					{
						netPlan.setRoutingType(RoutingType.HOP_BY_HOP_ROUTING);
						if (previousRoutingType != RoutingType.HOP_BY_HOP_ROUTING)
							updateNetPlanView();
					}
				}
			};		

			sourceRoutingActivated.addItemListener(itemRoutingTypeListener);
			hopByHopRoutingActivated.addItemListener(itemRoutingTypeListener);
		}

		routingSchemes = new ButtonGroup();
		routingSchemes.add(sourceRoutingActivated);
		routingSchemes.add(hopByHopRoutingActivated);
		
		txt_networkName = new JTextField();
		txt_networkDescription = new JTextArea();
		txt_networkDescription.setFont(new JLabel().getFont());
		txt_networkDescription.setLineWrap(true);
		txt_networkDescription.setWrapStyleWord(true);
		
		if (isEditable)
		{
			txt_layerName.setEditable(isEditable);
			txt_layerDescription.setEditable(isEditable);
			txt_layerDemandTrafficUnits.setEditable(isEditable);
			txt_layerLinkCapacityUnits.setEditable(isEditable);
			
			txt_networkName.setEditable(isEditable);
			txt_networkDescription.setEditable(isEditable);
			
			txt_networkName.getDocument().addDocumentListener(new DocumentAdapter()
			{
				@Override
				protected void updateInfo(String text)
				{
					if (isEditable) getDesign().setNetworkName(text);
				}
			});

			txt_networkDescription.getDocument().addDocumentListener(new DocumentAdapter()
			{
				@Override
				protected void updateInfo(String text)
				{
					if (isEditable) getDesign().setNetworkDescription(text);
				}
			});

			txt_layerName.getDocument().addDocumentListener(new DocumentAdapter()
			{
				@Override
				protected void updateInfo(String text)
				{
					long layerId = getDesign().getLayerDefaultId();
					if (isEditable)
					{
						JTable table = netPlanViewTable.get(NetworkElementType.LAYER);
						TableModel model = table.getModel();
						int numRows = model.getRowCount();
						for(int row = 0; row < numRows; row++)
						{
							if ((Long) model.getValueAt(row, 0) == layerId)
							{
								final DocumentListener me = this;

								final String previousValue = model.getValueAt(row, 1).toString();
								model.setValueAt(text, row, 1);
								if (!text.equals(previousValue))
								{
									SwingUtilities.invokeLater(new Runnable()
									{
										@Override
										public void run()
										{
											txt_layerName.getDocument().removeDocumentListener(me);
											txt_layerName.setText(previousValue);
											txt_layerName.getDocument().addDocumentListener(me);
										}
									});
								}
							}
						}
					}
				}
			});

			txt_layerLinkCapacityUnits.getDocument().addDocumentListener(new DocumentAdapter()
			{
				@Override
				protected void updateInfo(String text)
				{
					long layerId = getDesign().getLayerDefaultId();
					if (isEditable)
					{
						JTable table = netPlanViewTable.get(NetworkElementType.LAYER);
						TableModel model = table.getModel();
						int numRows = model.getRowCount();
						for(int row = 0; row < numRows; row++)
						{
							if ((Long) model.getValueAt(row, 0) == layerId)
							{
								final DocumentListener me = this;

								final String previousValue = model.getValueAt(row, 9).toString();
								model.setValueAt(text, row, 9);
								if (!text.equals(previousValue))
								{
									SwingUtilities.invokeLater(new Runnable()
									{
										@Override
										public void run()
										{
											txt_layerLinkCapacityUnits.getDocument().removeDocumentListener(me);
											txt_layerLinkCapacityUnits.setText(previousValue);
											txt_layerLinkCapacityUnits.getDocument().addDocumentListener(me);
										}
									});
								}
							}
						}
					}
				}
			});
			
			txt_layerDemandTrafficUnits.getDocument().addDocumentListener(new DocumentAdapter()
			{
				@Override
				protected void updateInfo(String text)
				{
					long layerId = getDesign().getLayerDefaultId();
					if (isEditable)
					{
						JTable table = netPlanViewTable.get(NetworkElementType.LAYER);
						TableModel model = table.getModel();
						int numRows = model.getRowCount();
						for(int row = 0; row < numRows; row++)
						{
							if ((Long) model.getValueAt(row, 0) == layerId)
							{
								final DocumentListener me = this;

								final String previousValue = model.getValueAt(row, 10).toString();
								model.setValueAt(text, row, 10);
								if (!text.equals(previousValue))
								{
									SwingUtilities.invokeLater(new Runnable()
									{
										@Override
										public void run()
										{
											txt_layerDemandTrafficUnits.getDocument().removeDocumentListener(me);
											txt_layerDemandTrafficUnits.setText(previousValue);
											txt_layerDemandTrafficUnits.getDocument().addDocumentListener(me);
										}
									});
								}
							}
						}
					}
				}
			});

			txt_layerDescription.getDocument().addDocumentListener(new DocumentAdapter()
			{
				@Override
				protected void updateInfo(String text)
				{
					long layerId = getDesign().getLayerDefaultId();
					if (isEditable)
					{
						JTable table = netPlanViewTable.get(NetworkElementType.LAYER);
						TableModel model = table.getModel();
						int numRows = model.getRowCount();
						for(int row = 0; row < numRows; row++)
						{
							if ((Long) model.getValueAt(row, 0) == layerId)
							{
								final DocumentListener me = this;

								final String previousValue = model.getValueAt(row, 8).toString();
								model.setValueAt(text, row, 8);
								if (!text.equals(previousValue))
								{
									SwingUtilities.invokeLater(new Runnable()
									{
										@Override
										public void run()
										{
											txt_layerDescription.getDocument().removeDocumentListener(me);
											txt_layerDescription.setText(previousValue);
											txt_layerDescription.getDocument().addDocumentListener(me);
										}
									});
								}
							}
						}
					}
				}
			});
		}

		String[] nodeTableHeader = netPlanViewTableHeader.get(NetworkElementType.NODE);
		TableModel nodeTableModel = new ClassAwareTableModel(new Object[1][nodeTableHeader.length], nodeTableHeader)
		{
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				if (getValueAt(rowIndex, columnIndex) == null) return false;
				
				if (columnIndex == 1) return true;
				if (!isEditable) return false;
				
				return columnIndex >= 2 && columnIndex <= 5;
			}

			@Override
			public void setValueAt(Object newValue, int row, int column)
			{
				Object oldValue = getValueAt(row, column);

				/* If value doesn't change, exit from function */
				if (newValue != null && newValue.equals(oldValue)) return;

				NetPlan netPlan = getDesign();

				if (getValueAt(row, 0) == null) row = row - 1;
				final long nodeId = (Long) getValueAt(row, 0);
				
				/* Perform checks, if needed */
				try
				{
					switch (column)
					{
						case 1:
							if (newValue == null) return;
							boolean visible = (Boolean) newValue;
							topologyPanel.getCanvas().setNodeVisible(nodeId, visible);
							break;
							
						case 2:
							netPlan.setNodeName(nodeId, newValue.toString());
							break;
							
						case 3:
							boolean isNodeUp = (Boolean) newValue;
							if (isNodeUp)
							{
								netPlan.setNodeUp(nodeId);
								Set<Long> layerIds = netPlan.getLayerIds();
								for(long layerId : layerIds)
								{
									if (netPlan.getRoutingType(layerId) == RoutingType.SOURCE_ROUTING)
									{
										netPlan.setAllProtectionSegmentsUp(layerId);
										netPlan.setAllRoutesUp(layerId);
									}
								}
								
								topologyPanel.getCanvas().setNodeUp(nodeId);
							}
							else
							{
								netPlan.setNodeDown(nodeId);
								topologyPanel.getCanvas().setNodeDown(nodeId);
							}

							updateNetPlanView();
							break;

						case 4:
						case 5:
							Point2D newPosition = column == 4 ? new Point2D.Double(Double.parseDouble(newValue.toString()), netPlan.getNodeXYPosition(nodeId).getY()) : new Point2D.Double(netPlan.getNodeXYPosition(nodeId).getX(), Double.parseDouble(newValue.toString()));
							netPlan.setNodeXYPosition(nodeId, newPosition.getX(), newPosition.getY());
							topologyPanel.getCanvas().setNodeXYPosition(nodeId, newPosition);
							topologyPanel.getCanvas().refresh();
							break;

						default:
							break;
					}
				}
				catch (Throwable ex)
				{
					ErrorHandling.showErrorDialog(ex.getMessage(), "Error modifying node");
					return;
				}

				/* Set new value */
				super.setValueAt(newValue, row, column);
			}
		};

		String[] linkTableHeader = netPlanViewTableHeader.get(NetworkElementType.LINK);
		TableModel linkTableModel = new ClassAwareTableModel(new Object[1][linkTableHeader.length], linkTableHeader)
		{
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				if (getValueAt(rowIndex, columnIndex) == null) return false;
				if (columnIndex == 1) return true;
				if (!isEditable) return false;
				
				return columnIndex == 4 || columnIndex == 6 || columnIndex == 12 || columnIndex == 13;
			}

			@Override
			public void setValueAt(Object newValue, int row, int column)
			{
				Object oldValue = getValueAt(row, column);

				/* If value doesn't change, exit from function */
				if (newValue != null && newValue.equals(oldValue)) return;

				NetPlan netPlan = getDesign();

				if (getValueAt(row, 0) == null) row = row - 1;
				final long linkId = (Long) getValueAt(row, 0);
				
				/* Perform checks, if needed */
				try
				{
					switch (column)
					{
						case 1:
							if (newValue == null) return;
							boolean visible = (Boolean) newValue;
							topologyPanel.getCanvas().setLinkVisible(linkId, visible);
							break;

						case 4:
							boolean isLinkUp = (Boolean) newValue;
							if (isLinkUp)
							{
								netPlan.setLinkUp(linkId);
								if (netPlan.getRoutingType() == RoutingType.SOURCE_ROUTING)
								{
									netPlan.setAllProtectionSegmentsUp();
									netPlan.setAllRoutesUp();
								}
								
								topologyPanel.getCanvas().setLinkUp(linkId);
							}
							else
							{
								netPlan.setLinkDown(linkId);
								topologyPanel.getCanvas().setLinkDown(linkId);
							}
							
							updateNetPlanView();
							break;

						case 6:
							netPlan.setLinkCapacity(linkId, Double.parseDouble(newValue.toString()));
							updateNetPlanView();
							break;

						case 12:
							netPlan.setLinkLengthInKm(linkId, Double.parseDouble(newValue.toString()));
							updateNetPlanView();
							break;

						case 13:
							netPlan.setLinkPropagationSpeedInKmPerSecond(linkId, Double.parseDouble(newValue.toString()));
							updateNetPlanView();
							break;

						default:
							break;
					}
				}
				catch (Throwable ex)
				{
					ErrorHandling.showErrorDialog(ex.getMessage(), "Error modifying link");
					return;
				}

				/* Set new value */
				super.setValueAt(newValue, row, column);
			}
		};

		String[] demandTableHeader = netPlanViewTableHeader.get(NetworkElementType.DEMAND);
		TableModel demandTableModel = new ClassAwareTableModel(new Object[1][demandTableHeader.length], demandTableHeader)
		{
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				if (getValueAt(rowIndex, columnIndex) == null) return false;
				if (!isEditable) return false;

				return columnIndex == 4;
			}

			@Override
			public void setValueAt(Object newValue, int row, int column)
			{
				Object oldValue = getValueAt(row, column);

				/* If value doesn't change, exit from function */
				if (newValue.equals(oldValue)) return;

				NetPlan netPlan = getDesign();

				if (getValueAt(row, 0) == null) row = row - 1;
				final long demandId = (Long) getValueAt(row, 0);
				
				/* Perform checks, if needed */
				try
				{
					switch (column)
					{
						case 4:
							netPlan.setDemandOfferedTraffic(demandId, Double.parseDouble(newValue.toString()));

							updateNetPlanView();
							break;

						default:
							break;
					}
				}
				catch (Throwable ex)
				{
					ErrorHandling.showErrorDialog(ex.getMessage(), "Error modifying demand");
					return;
				}

				/* Set new value */
				super.setValueAt(newValue, row, column);
			}
		};

		String[] routeTableHeader = netPlanViewTableHeader.get(NetworkElementType.ROUTE);
		TableModel routeTableModel = new ClassAwareTableModel(new Object[1][routeTableHeader.length], routeTableHeader)
		{
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				if (getValueAt(rowIndex, columnIndex) == null) return false;
				if (!isEditable) return false;
				
				return columnIndex == 5 || columnIndex == 6;
			}

			@Override
			public void setValueAt(Object newValue, int row, int column)
			{
				Object oldValue = getValueAt(row, column);

				if (newValue.equals(oldValue)) return;

				NetPlan netPlan = getDesign();

				if (getValueAt(row, 0) == null) row = row - 1;
				final long routeId = (Long) getValueAt(row, 0);
				
				/* Perform checks, if needed */
				try
				{
					switch (column)
					{
						case 5:
							netPlan.setRouteCarriedTraffic(routeId, Double.parseDouble(newValue.toString()));
							updateNetPlanView();
							break;

						case 6:
							netPlan.setRouteOccupiedCapacity(routeId, Double.parseDouble(newValue.toString()));
							updateNetPlanView();
							break;

						default:
							break;
					}
				}
				catch (Throwable ex)
				{
					ErrorHandling.showErrorDialog(ex.getMessage(), "Error modifying route");
					return;
				}

				/* Set new value */
				super.setValueAt(newValue, row, column);
			}
		};

		String[] segmentTableHeader = netPlanViewTableHeader.get(NetworkElementType.PROTECTION_SEGMENT);
		TableModel segmentTableModel = new ClassAwareTableModel(new Object[1][segmentTableHeader.length], segmentTableHeader)
		{
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				if (getValueAt(rowIndex, columnIndex) == null) return false;
				if (!isEditable) return false;
				
				return columnIndex == 3;
			}

			@Override
			public void setValueAt(Object newValue, int row, int column)
			{
				Object oldValue = getValueAt(row, column);

				if (newValue.equals(oldValue)) return;

				NetPlan netPlan = getDesign();

				if (getValueAt(row, 0) == null) row = row - 1;
				final long segmentId = (Long) getValueAt(row, 0);
				
				/* Perform checks, if needed */
				try
				{
					switch (column)
					{
						case 3:
							netPlan.setProtectionSegmentReservedCapacity(segmentId, Double.parseDouble(newValue.toString()));
							updateNetPlanView();
							break;

						default:
							break;
					}
				}
				catch (Throwable ex)
				{
					ErrorHandling.showErrorDialog(ex.getMessage(), "Error modifying protection segment");
					return;
				}

				/* Set new value */
				super.setValueAt(newValue, row, column);
			}
		};

		String[] forwardingRuleTableHeader = netPlanViewTableHeader.get(NetworkElementType.FORWARDING_RULE);
		TableModel forwardingRuleTableModel = new ClassAwareTableModel(new Object[1][forwardingRuleTableHeader.length], forwardingRuleTableHeader)
		{
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				if (getValueAt(rowIndex, columnIndex) == null) return false;
				if (!isEditable) return false;
				
				return columnIndex == 3;
			}

			@Override
			public void setValueAt(Object newValue, int row, int column)
			{
				Object oldValue = getValueAt(row, column);

				if (newValue.equals(oldValue)) return;

				NetPlan netPlan = getDesign();

				if (getValueAt(row, 0) == null) row = row - 1;
				final Pair<Long, Long> forwardingRule = Pair.of((Long) getValueAt(row, 1), (Long) getValueAt(row, 2));
				
				/* Perform checks, if needed */
				try
				{
					switch (column)
					{
						case 3:
							netPlan.setForwardingRuleSplittingRatio(forwardingRule, Double.parseDouble(newValue.toString()));
							updateNetPlanView();
							break;

						default:
							break;
					}
				}
				catch (Throwable ex)
				{
					ErrorHandling.showErrorDialog(ex.getMessage(), "Error modifying forwarding rule");
					return;
				}

				/* Set new value */
				super.setValueAt(newValue, row, column);
			}
		};

		String[] srgTableHeader = netPlanViewTableHeader.get(NetworkElementType.SRG);
		TableModel srgTableModel = new ClassAwareTableModel(new Object[1][srgTableHeader.length], srgTableHeader)
		{
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				if (getValueAt(rowIndex, columnIndex) == null) return false;
				if (!isEditable) return false;
				
				return columnIndex >= 1 && columnIndex <= 4;
			}

			@Override
			public void setValueAt(Object newValue, int row, int column)
			{
				Object oldValue = getValueAt(row, column);

				if (newValue.equals(oldValue)) return;

				NetPlan netPlan = getDesign();
				NetPlan aux_netPlan = netPlan.copy();

				if (getValueAt(row, 0) == null) row = row - 1;
				final long srgId = (Long) getValueAt(row, 0);
				
				/* Perform checks, if needed */
				try
				{
					switch (column)
					{
						case 1:
							netPlan.setSRGMeanTimeToFailInHours(srgId, Double.parseDouble(newValue.toString()));
							break;

						case 2:
							netPlan.setSRGMeanTimeToRepairInHours(srgId, Double.parseDouble(newValue.toString()));
							break;

						case 3:
							Set<Long> previousNodeIds = new LinkedHashSet<Long>(netPlan.getSRGNodes(srgId));
							for (long nodeId : previousNodeIds) netPlan.removeNodeFromSRG(nodeId, srgId);

							long[] nodeIds = StringUtils.toLongArray(StringUtils.split(newValue.toString(), ", "));
							for (long nodeId : nodeIds) netPlan.addNodeToSRG(nodeId, row);

							Arrays.sort(nodeIds);
							newValue = LongUtils.join(nodeIds, ", ");

							break;

						case 4:
							Set<Long> previousLinkIds = new LinkedHashSet<Long>(netPlan.getSRGLinks(srgId));
							for (long linkId : previousLinkIds) netPlan.removeLinkFromSRG(linkId, srgId);

							long[] linkIds = StringUtils.toLongArray(StringUtils.split(newValue.toString(), ", "));
							for (long linkId : linkIds) netPlan.addLinkToSRG(linkId, row);

							Arrays.sort(linkIds);
							newValue = LongUtils.join(linkIds, ", ");
							break;

						default:
							break;
					}
				}
				catch (Throwable ex)
				{
					getDesign().assignFrom(aux_netPlan);
					ErrorHandling.showErrorDialog(ex.getMessage(), "Error modifying SRG");
					return;
				}

				/* Set new value */
				super.setValueAt(newValue, row, column);
			}
		};

		String[] layerTableHeader = netPlanViewTableHeader.get(NetworkElementType.LAYER);
		TableModel layerTableModel = new ClassAwareTableModel(new Object[1][layerTableHeader.length], layerTableHeader)
		{
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				if (getValueAt(rowIndex, columnIndex) == null) return false;
				if (!isEditable) return false;
				
				return columnIndex == 1 || columnIndex == 8 || columnIndex == 9 || columnIndex == 10;
			}

			@Override
			public void setValueAt(Object newValue, int row, int column)
			{
				Object oldValue = getValueAt(row, column);

				/* If value doesn't change, exit from function */
				if (newValue != null && newValue.equals(oldValue)) return;

				NetPlan netPlan = getDesign();

				if (getValueAt(row, 0) == null) row = row - 1;
				final long layerId = (Long) getValueAt(row, 0);
				
				/* Perform checks, if needed */
				try
				{
					switch (column)
					{
						case 1:
							netPlan.setLayerName(layerId, newValue.toString());
							updateLayerChooser();
							break;
							
						case 8:
							netPlan.setLayerDescription(layerId, newValue.toString());
							break;

						case 9:
							netPlan.setLinkCapacityUnitsName(layerId, newValue.toString());
							break;

						case 10:
							netPlan.setDemandTrafficUnitsName(layerId, newValue.toString());
							break;

						default:
							break;
					}
				}
				catch (Throwable ex)
				{
					ErrorHandling.showErrorDialog(ex.getMessage(), "Error modifying layer");
					return;
				}

				/* Set new value */
				super.setValueAt(newValue, row, column);
			}
		};
		
		TableModel networkAttributeTableModel = new ClassAwareTableModel(new Object[1][attributeTableHeader.length], attributeTableHeader);
		TableModel layerAttributeTableModel = new ClassAwareTableModel(new Object[1][attributeTableHeader.length], attributeTableHeader);
		
		netPlanViewTable.put(NetworkElementType.NODE, new AdvancedJTable(nodeTableModel));
		netPlanViewTable.put(NetworkElementType.LINK, new AdvancedJTable(linkTableModel));
		netPlanViewTable.put(NetworkElementType.DEMAND, new AdvancedJTable(demandTableModel));
		netPlanViewTable.put(NetworkElementType.ROUTE, new AdvancedJTable(routeTableModel));
		netPlanViewTable.put(NetworkElementType.PROTECTION_SEGMENT, new AdvancedJTable(segmentTableModel));
		netPlanViewTable.put(NetworkElementType.FORWARDING_RULE, new AdvancedJTable(forwardingRuleTableModel));
		netPlanViewTable.put(NetworkElementType.SRG, new AdvancedJTable(srgTableModel));
		netPlanViewTable.put(NetworkElementType.LAYER, new AdvancedJTable(layerTableModel));
		
		networkAttributeTable = new AdvancedJTable(networkAttributeTableModel);
		layerAttributeTable = new AdvancedJTable(layerAttributeTableModel);
		
		for(Entry<NetworkElementType, JTable> entry : netPlanViewTable.entrySet())
		{
			NetworkElementType type = entry.getKey();
			JTable table = entry.getValue();
			
			table.setDefaultRenderer(Boolean.class, new CellRenderers.CheckBoxRenderer());
			table.setDefaultRenderer(Double.class, new NumberCellRenderer());
			table.setDefaultRenderer(Object.class, new CellRenderers.NonEditableCellRenderer());
			table.setDefaultRenderer(Float.class, new NumberCellRenderer());
			table.setDefaultRenderer(Long.class, new CellRenderers.NumberCellRenderer());
			table.setDefaultRenderer(Integer.class, new CellRenderers.NumberCellRenderer());
			table.setDefaultRenderer(String.class, new CellRenderers.NonEditableCellRenderer());
			
			if (table == netPlanViewTable.get(NetworkElementType.DEMAND))
			{
				table.getColumnModel().getColumn(table.convertColumnIndexToView(6)).setCellRenderer(new CellRenderers.LostTrafficCellRenderer(4));
			}
			else if (table == netPlanViewTable.get(NetworkElementType.LINK))
			{
				table.setDefaultRenderer(Boolean.class, new CellRenderers.LinkRenderer(table.getDefaultRenderer(Boolean.class), this));
				table.setDefaultRenderer(Double.class, new CellRenderers.LinkRenderer(table.getDefaultRenderer(Double.class), this));
				table.setDefaultRenderer(Object.class, new CellRenderers.LinkRenderer(table.getDefaultRenderer(Object.class), this));
				table.setDefaultRenderer(Float.class, new CellRenderers.LinkRenderer(table.getDefaultRenderer(Float.class), this));
				table.setDefaultRenderer(Long.class, new CellRenderers.LinkRenderer(table.getDefaultRenderer(Long.class), this));
				table.setDefaultRenderer(Integer.class, new CellRenderers.LinkRenderer(table.getDefaultRenderer(Integer.class), this));
				table.setDefaultRenderer(String.class, new CellRenderers.LinkRenderer(table.getDefaultRenderer(String.class), this));
			}
			else if (table == netPlanViewTable.get(NetworkElementType.ROUTE))
			{
				table.setDefaultRenderer(Boolean.class, new CellRenderers.RouteRenderer(table.getDefaultRenderer(Boolean.class), this));
				table.setDefaultRenderer(Double.class, new CellRenderers.RouteRenderer(table.getDefaultRenderer(Double.class), this));
				table.setDefaultRenderer(Object.class, new CellRenderers.RouteRenderer(table.getDefaultRenderer(Object.class), this));
				table.setDefaultRenderer(Float.class, new CellRenderers.RouteRenderer(table.getDefaultRenderer(Float.class), this));
				table.setDefaultRenderer(Long.class, new CellRenderers.RouteRenderer(table.getDefaultRenderer(Long.class), this));
				table.setDefaultRenderer(Integer.class, new CellRenderers.RouteRenderer(table.getDefaultRenderer(Integer.class), this));
				table.setDefaultRenderer(String.class, new CellRenderers.RouteRenderer(table.getDefaultRenderer(String.class), this));
			}
			else if (table == netPlanViewTable.get(NetworkElementType.FORWARDING_RULE))
			{
				table.setDefaultRenderer(Boolean.class, new CellRenderers.ForwardingRuleRenderer(table.getDefaultRenderer(Boolean.class), this));
				table.setDefaultRenderer(Double.class, new CellRenderers.ForwardingRuleRenderer(table.getDefaultRenderer(Double.class), this));
				table.setDefaultRenderer(Object.class, new CellRenderers.ForwardingRuleRenderer(table.getDefaultRenderer(Object.class), this));
				table.setDefaultRenderer(Float.class, new CellRenderers.ForwardingRuleRenderer(table.getDefaultRenderer(Float.class), this));
				table.setDefaultRenderer(Long.class, new CellRenderers.ForwardingRuleRenderer(table.getDefaultRenderer(Long.class), this));
				table.setDefaultRenderer(Integer.class, new CellRenderers.ForwardingRuleRenderer(table.getDefaultRenderer(Integer.class), this));
				table.setDefaultRenderer(String.class, new CellRenderers.ForwardingRuleRenderer(table.getDefaultRenderer(String.class), this));
			}
			else if (table == netPlanViewTable.get(NetworkElementType.NODE) || table == netPlanViewTable.get(NetworkElementType.PROTECTION_SEGMENT))
			{
				table.setDefaultRenderer(Boolean.class, new CellRenderers.UpDownRenderer(table.getDefaultRenderer(Boolean.class), this, type));
				table.setDefaultRenderer(Double.class, new CellRenderers.UpDownRenderer(table.getDefaultRenderer(Double.class), this, type));
				table.setDefaultRenderer(Object.class, new CellRenderers.UpDownRenderer(table.getDefaultRenderer(Object.class), this, type));
				table.setDefaultRenderer(Float.class, new CellRenderers.UpDownRenderer(table.getDefaultRenderer(Float.class), this, type));
				table.setDefaultRenderer(Long.class, new CellRenderers.UpDownRenderer(table.getDefaultRenderer(Long.class), this, type));
				table.setDefaultRenderer(Integer.class, new CellRenderers.UpDownRenderer(table.getDefaultRenderer(Integer.class), this, type));
				table.setDefaultRenderer(String.class, new CellRenderers.UpDownRenderer(table.getDefaultRenderer(String.class), this, type));
			}
		}
		
		netPlanView = new JTabbedPane();
//		netPlanView.setFocusable(false);

		KeyListener cursorNavigation = new TableCursorNavigation();
		
		for(NetworkElementType elementType : Constants.NetworkElementType.values())
		{
			if (elementType == NetworkElementType.NETWORK)
			{
				String[] columnTips = attributeTableTips;
				String[] columnHeader = attributeTableHeader;

				ColumnHeaderToolTips tips = new ColumnHeaderToolTips();
				for (int c = 0; c < columnHeader.length; c++)
				{
					TableColumn col = networkAttributeTable.getColumnModel().getColumn(c);
					tips.setToolTip(col, columnTips[c]);
				}

				networkAttributeTable.getTableHeader().addMouseMotionListener(tips);
				networkAttributeTable.setAutoCreateRowSorter(true);

				JScrollPane scrollPane = new JScrollPane(networkAttributeTable);
				ScrollPaneLayout layout = new FullScrollPaneLayout();
				scrollPane.setLayout(layout);
				scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
				
				JTable layerTable = netPlanViewTable.get(NetworkElementType.LAYER);
				String[] columnTips1 = netPlanViewTableTips.get(NetworkElementType.LAYER);
				String[] columnHeader1 = netPlanViewTableHeader.get(NetworkElementType.LAYER);

				ColumnHeaderToolTips tips1 = new ColumnHeaderToolTips();
				for (int c = 0; c < columnHeader1.length; c++)
				{
					TableColumn col = layerTable.getColumnModel().getColumn(c);
					tips1.setToolTip(col, columnTips1[c]);
				}

				layerTable.getTableHeader().addMouseMotionListener(tips1);

				if (allowShowInitialNetPlan()) layerTable.setRowSorter(new CurrentAndPlannedStateTableSorter(layerTable.getModel()));
				else layerTable.setAutoCreateRowSorter(true);
				
				JScrollPane scrollPane1 = new JScrollPane(layerTable);
				ScrollPaneLayout layout1 = new FullScrollPaneLayout();
				scrollPane1.setLayout(layout1);
				scrollPane1.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
				
				MouseAdapter tableAdapter = new PopupMenuNetPlan(this, topologyPanel, layerTable.getModel(), NetworkElementType.LAYER, isEditable);
				layerTable.addMouseListener(tableAdapter);

				FixedColumnDecorator decorator = new FixedColumnDecorator(scrollPane1, 1);
				decorator.getFixedTable().getColumnModel().getColumn(0).setMinWidth(50);

				netPlanViewTableDecorator.put(NetworkElementType.LAYER, decorator);
				netPlanViewTableComponent.put(NetworkElementType.LAYER, scrollPane1);
				
				JPanel networkPane = new JPanel(new MigLayout("", "[][grow]", "[][][grow][][][][][grow]"));
				networkPane.add(new JLabel("Name"));
				networkPane.add(txt_networkName, "grow, wrap");
				networkPane.add(new JLabel("Description"), "aligny top");
				networkPane.add(new JScrollPane(txt_networkDescription), "grow, wrap, height 100::");
				networkPane.add(scrollPane, "grow, spanx 2, wrap");
				networkPane.add(new JLabel("Number of layers"), "grow");
				networkPane.add(txt_numLayers, "grow, wrap");
				networkPane.add(new JLabel("Number of nodes"), "grow");
				networkPane.add(txt_numNodes, "grow, wrap");
				networkPane.add(new JLabel("Number of SRGs"), "grow");
				networkPane.add(txt_numSRGs, "grow, wrap");
				networkPane.add(new JLabel("Layer information"), "grow, spanx2, wrap");
				networkPane.add(scrollPane1, "grow, spanx 2");
				netPlanViewTableComponent.put(elementType, networkPane);
				networkAttributeTable.addKeyListener(cursorNavigation);
			}
			else
			{
				JTable table = elementType == NetworkElementType.LAYER ? layerAttributeTable : netPlanViewTable.get(elementType);
				String[] columnTips = elementType == NetworkElementType.LAYER ? attributeTableTips : netPlanViewTableTips.get(elementType);
				String[] columnHeader = elementType == NetworkElementType.LAYER ? attributeTableHeader : netPlanViewTableHeader.get(elementType);

				ColumnHeaderToolTips tips = new ColumnHeaderToolTips();
				for (int c = 0; c < columnHeader.length; c++)
				{
					TableColumn col = table.getColumnModel().getColumn(c);
					tips.setToolTip(col, columnTips[c]);
				}

				table.getTableHeader().addMouseMotionListener(tips);

				JScrollPane scrollPane = new JScrollPane(table);
				ScrollPaneLayout layout = new FullScrollPaneLayout();
				scrollPane.setLayout(layout);
				scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
				
				switch(elementType)
				{
					case NETWORK:
						break;

					case LAYER:
						layerSummaryTables = new ParamValueTable[4];
						for (int i = 0; i < layerSummaryTables.length; i++)
						{
							layerSummaryTables[i] = new ParamValueTable(layerSummaryTableHeader);
							layerSummaryTables[i].setAutoCreateRowSorter(true);
							layerSummaryTables[i].addKeyListener(cursorNavigation);
						}

						table.addKeyListener(cursorNavigation);

						layerMetricsInfo = new JPanel();
						layerMetricsInfo.setLayout(new MigLayout("insets 0 0 0 0", "[grow]"));
						layerMetricsInfo.add(new JLabel("Topology and link capacities"), "growx, wrap");
						layerMetricsInfo.add(new JScrollPane(layerSummaryTables[0]), "growx, wrap");
						layerMetricsInfo.add(new JLabel("Traffic"), "growx, wrap");
						layerMetricsInfo.add(new JScrollPane(layerSummaryTables[1]), "growx, wrap");
						layerMetricsInfo.add(new JLabel("Routing"), "growx, wrap");
						layerMetricsInfo.add(new JScrollPane(layerSummaryTables[2]), "growx, wrap");
						layerMetricsInfo.add(new JLabel("Resilience information"), "growx, wrap");
						layerMetricsInfo.add(new JScrollPane(layerSummaryTables[3]), "growx");

						JPanel layerPane = new JPanel(new MigLayout("", "[][grow]", "[][][][][][grow]"));
						layerPane.add(new JLabel("Name"));
						layerPane.add(txt_layerName, "grow, wrap");
						layerPane.add(new JLabel("Description"), "aligny top");
						layerPane.add(new JScrollPane(txt_layerDescription), "grow, wrap, height 100::");
						layerPane.add(new JLabel("Link capacity units"));
						layerPane.add(txt_layerLinkCapacityUnits, "grow, wrap");
						layerPane.add(new JLabel("Demand traffic units"));
						layerPane.add(txt_layerDemandTrafficUnits, "grow, wrap");
						layerPane.add(new JLabel("Routing type"));
						
						JPanel radioPanel = new JPanel(new GridLayout(0, 1));
				        radioPanel.add(sourceRoutingActivated);
						radioPanel.add(hopByHopRoutingActivated);
						layerPane.add(radioPanel, "grow, wrap");
						
						layerPane.add(scrollPane, "grow, spanx 2");
						JScrollPane layerInfoScrollPane = new JScrollPane(layerMetricsInfo);
						layerInfoScrollPane.setBorder(BorderFactory.createEmptyBorder());
						
						forceUpdate = new JButton("Update all metrics");
						forceUpdate.addActionListener(new ActionListener()
						{
							@Override
							public void actionPerformed(ActionEvent e)
							{
								updateLayerMetrics(getDesign(), true);
							}
						});
						
						forceUpdate.setToolTipText("Use this button to update non-computed metrics due to hard computations");
						forceUpdate.setVisible(false);

						JPanel layerInfoPane = new JPanel(new BorderLayout());
						layerInfoPane.add(forceUpdate, BorderLayout.NORTH);
						layerInfoPane.add(layerInfoScrollPane, BorderLayout.CENTER);

						JSplitPane splitPaneTopology = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
						splitPaneTopology.setTopComponent(layerPane);
						splitPaneTopology.setBottomComponent(layerInfoPane);
						splitPaneTopology.setResizeWeight(0.3);
						splitPaneTopology.addPropertyChangeListener(new ProportionalResizeJSplitPaneListener());
						netPlanViewTableComponent.put(elementType, splitPaneTopology);
						break;
						
					default:
						MouseAdapter tableAdapter = new PopupMenuNetPlan(this, topologyPanel, table.getModel(), elementType, isEditable);
						table.addMouseListener(tableAdapter);

						int numColums;
						switch(elementType)
						{
							case NODE:
							case LINK:
								numColums = 2;
								break;
								
							case FORWARDING_RULE:
								numColums = 3;
								break;
								
							default:
								numColums = 1;
						}

						if (allowShowInitialNetPlan()) table.setRowSorter(new CurrentAndPlannedStateTableSorter(table.getModel()));
						else table.setAutoCreateRowSorter(true);
						
						FixedColumnDecorator decorator = new FixedColumnDecorator(scrollPane, numColums);
						decorator.getFixedTable().getColumnModel().getColumn(0).setMinWidth(50);
						netPlanViewTableDecorator.put(elementType, decorator);
						netPlanViewTableComponent.put(elementType, scrollPane);
						
						break;
				}
			}
		}

		JPanel pane = new JPanel(new BorderLayout());
		pane.add(netPlanView, BorderLayout.CENTER);
		
		if (allowShowInitialNetPlan())
		{
			showInitialPlan = new JCheckBox("Toggle show/hide planning information", true);
			showInitialPlan.addItemListener(new ItemListener()
			{
				@Override
				public void itemStateChanged(ItemEvent e)
				{
					RowFilter<TableModel, Integer> rowFilter = e.getStateChange() == ItemEvent.SELECTED ? null : new RowFilter<TableModel, Integer>()
					{
						@Override
						public boolean include(RowFilter.Entry<? extends TableModel, ? extends Integer> entry)
						{
							if (entry.getIdentifier() == 0) return true;
							
							if (entry.getValue(0) instanceof CurrentAndPlannedStateTableCellValue)
								return ((CurrentAndPlannedStateTableCellValue) entry.getValue(0)).value != null;
							else
								return entry.getValue(0) != null;
						}
					};
					
					for (NetworkElementType elementType : Constants.NetworkElementType.values())
					{
						if (elementType == NetworkElementType.NETWORK) continue;
						
						((TableRowSorter) netPlanViewTable.get(elementType).getRowSorter()).setRowFilter(rowFilter);
					}
				}
			});

			showInitialPlan.setSelected(false);
			pane.add(showInitialPlan, BorderLayout.NORTH);
		}

		addTab(isEditable ? "View/edit network state" : "View network state", pane);
		viewNetPlanTabIndex = 0;
	}
	
	private void configureReportPane()
	{
		reportController = new ThreadExecutionController(this);

		File REPORTS_DIRECTORY = new File(CURRENT_DIR + SystemUtils.getDirectorySeparator() + "workspace");
		REPORTS_DIRECTORY = REPORTS_DIRECTORY.isDirectory() ? REPORTS_DIRECTORY : CURRENT_DIR;
		ParameterValueDescriptionPanel reportParameters = new ParameterValueDescriptionPanel();
		reportSelector = new RunnableSelector("Report", null, IReport.class, REPORTS_DIRECTORY, reportParameters);
		reportPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		reportContainer = new JTabbedPane();
//		reportContainer.setFocusable(false);

		final JPanel pnl_buttons = new JPanel(new MigLayout("", "[center, grow]", "[]"));

		reportContainer.setVisible(false);
		
		addKeyCombinationAction("Close selected report", new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				int tab = reportContainer.getSelectedIndex();
				if (tab == -1) return;

				reportContainer.remove(tab);
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK));
		
		reportContainer.addContainerListener(new ContainerListener()
		{
			@Override
			public void componentAdded(ContainerEvent e)
			{
				reportContainer.setVisible(true);
				reportPane.setDividerLocation(0.5);
			}

			@Override
			public void componentRemoved(ContainerEvent e)
			{
				if (reportContainer.getTabCount() == 0)
				{
					reportContainer.setVisible(false);
				}
			}
		});

		JButton btn_show = new JButton("Show");
		btn_show.setToolTipText("Show the report");
		btn_show.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				reportController.execute();
			}
		});

		reportContainer.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseClicked(MouseEvent e)
			{
				int tabNumber = reportContainer.getUI().tabForCoordinate(reportContainer, e.getX(), e.getY());

				if (tabNumber >= 0)
				{
					Rectangle rect = ((TabIcon) reportContainer.getIconAt(tabNumber)).getBounds();
					if (rect.contains(e.getX(), e.getY())) reportContainer.removeTabAt(tabNumber);
				}
			}
		});

		pnl_buttons.add(btn_show);

		JPanel pane = new JPanel(new BorderLayout());
		pane.add(reportSelector, BorderLayout.CENTER);
		pane.add(pnl_buttons, BorderLayout.SOUTH);
		reportPane.setTopComponent(pane);

		reportPane.setBottomComponent(reportContainer);
		reportPane.addPropertyChangeListener(new ProportionalResizeJSplitPaneListener());
		reportPane.setResizeWeight(0.5);
		
		addTab("View reports", reportPane);
	}
	
	/**
	 * Allows to include custom code after initializing the topologyPanel panel (i.e. add new plugins).
	 * 
	 * @since 0.3.0
	 */
	protected void configureTopologyPanel()
	{
		popupPlugin = new PopupEditGraphPlugin(this, MouseEvent.BUTTON3_MASK);
		
		getTopologyPanel().addPlugin(new PanGraphPlugin(this, MouseEvent.BUTTON1_MASK));
		getTopologyPanel().addPlugin(new AddLinkGraphPlugin(this, MouseEvent.BUTTON1_MASK, MouseEvent.BUTTON1_MASK | MouseEvent.SHIFT_MASK));
		getTopologyPanel().addPlugin(popupPlugin);
		getTopologyPanel().addPlugin(new MoveNodePlugin(this, MouseEvent.BUTTON1_MASK | MouseEvent.CTRL_MASK));
	}
	
	/**
	 * Returns a reference to the topologyPanel panel.
	 * 
	 * @return Reference to the topologyPanel panel
	 * @since 0.3.0
	 */
	protected final TopologyPanel getTopologyPanel()
	{
		return topologyPanel;
	}
	
	/**
	 * Allows to execute some action whenever a layer is selected in the GUI.
	 * 
	 * @param layerId Layer identifier
	 * @since 0.3.0
	 */
	protected void layerChanged(long layerId)
	{
	}

	/**
	 * Allows to include custom code after resetting the topologyPanel panel.
	 * 
	 * @since 0.3.0
	 */
	protected void reset_internal()
	{
		loadDesign(new NetPlan());
	}
	
	private void selectLayer(long layerId)
	{
		long currentLayerId = (Long) ((StringLabeller) layerChooser.getSelectedItem()).getObject();
		if (layerId == currentLayerId) return;
		
		int numLayers = layerChooser.getItemCount();
		for(int l = 0; l < numLayers; l++)
		{
			if (layerId == (Long) ((StringLabeller) layerChooser.getItemAt(l)).getObject())
			{
				layerChooser.setSelectedItem(layerChooser.getItemAt(l));
				return;
			}
		}
		
		throw new RuntimeException("Bad");
	}
	
	/**
	 * Shows the tab corresponding associated to a network element.
	 * 
	 * @param type Network element type
	 * @param itemId Item identifier (if null, it will just show the tab)
	 * @since 0.3.0
	 */
	protected void selectNetPlanViewItem(NetworkElementType type, Object itemId)
	{
		long layerId = getDesign().getLayerDefaultId();
		selectNetPlanViewItem(layerId, type, itemId);
	}
	
	/**
	 * Shows the tab corresponding associated to a network element.
	 * 
	 * @param layerId Layer identifier
	 * @param type Network element type
	 * @param itemId Item identifier (if null, it will just show the tab)
	 * @since 0.3.0
	 */
	protected void selectNetPlanViewItem(long layerId, NetworkElementType type, Object itemId)
	{
		selectLayer(layerId);
		showTab(viewNetPlanTabIndex);
		
		JTable table = netPlanViewTable.get(type);
		int tabIndex = netPlanView.getSelectedIndex();
		int col = 0;
		if (netPlanView.getTitleAt(tabIndex).equals(netPlanViewTabName.get(type)))
		{
			col = table.getSelectedColumn();
			if (col == -1) col = 0;
		}
		else
		{
			netPlanView.setSelectedComponent(netPlanViewTableComponent.get(type));
		}
		
		table.clearSelection();
		
		if (itemId == null) return;

		TableModel model = table.getModel();
		int numRows = model.getRowCount();
		for(int row = 0; row < numRows; row++)
		{
			Object obj = model.getValueAt(row, 0);
			if (obj == null) continue;

			if (type == NetworkElementType.FORWARDING_RULE)
			{
				obj = Pair.of(Long.parseLong(model.getValueAt(row, 1).toString().split(" ")[0]), Long.parseLong(model.getValueAt(row, 2).toString().split(" ")[0]));
				if (!obj.equals(itemId)) continue;
			}
			else if ((long) obj != (long) itemId)
			{
				continue;
			}

			row = table.convertRowIndexToView(row);
			table.changeSelection(row, col, false, true);
			return;
		}
		
		throw new RuntimeException(type + " " + itemId + " does not exist");
	}
	
	/**
	 * Allows to include actions when a {@code NetPlan} object is loaded.
	 * 
	 * @param netPlan {@code NetPlan} object
	 * @since 0.3.0
	 */
	protected void setNetPlan(NetPlan netPlan)
	{
		currentNetPlan = netPlan;
		if (allowShowInitialNetPlan()) initialNetPlan = currentNetPlan.copy();
	}

	/**
	 * Shows the <code>NetPlan</code> view, moving to the corresponding tab.
	 * 
	 * @since 0.3.0
	 */
	protected final void showNetPlanView()
	{
		netPlanView.setSelectedIndex(0);
		showTab(viewNetPlanTabIndex);
	}
	
	/**
	 * Shows the desired tab in <code>NetPlan</code> view.
	 * 
	 * @param tabIndex Tab index
	 * @since 0.3.0
	 */
	protected final void showTab(int tabIndex)
	{
		if (tabIndex < rightPane.getTabCount() && rightPane.getSelectedIndex() != tabIndex) 		
		{
			rightPane.setSelectedIndex(tabIndex);
			rightPane.requestFocusInWindow();
		}
	}
	
	private void updateLayerMetrics(NetPlan netPlan, boolean applyHardComputations)
	{
		Set<Long> nodeIds = netPlan.getNodeIds();
		Set<Long> linkIds = netPlan.getLinkIds();
		int N = netPlan.getNumberOfNodes();
		int E = netPlan.getNumberOfLinks();
		int D = netPlan.getNumberOfDemands();
		int numSRGs = netPlan.getNumberOfSRGs();
		double U_e = netPlan.getLinkTotalCapacity();
		double H_d = netPlan.getDemandTotalOfferedTraffic();
		double u_e_avg = E == 0 ? 0 : U_e / E;

		int E_limitedCapacityLinks = 0;
		double totalCapacityInstalled_limitedCapacityLinks = 0;
		for(long linkId : linkIds)
		{
			double u_e = netPlan.getLinkCapacity(linkId);
			if (u_e < Double.MAX_VALUE)
			{
				E_limitedCapacityLinks++;
				totalCapacityInstalled_limitedCapacityLinks += u_e;
			}
		}

		double averageTotalCapacityInstalled_limitedCapacityLinks = E_limitedCapacityLinks == 0 ? 0 : totalCapacityInstalled_limitedCapacityLinks / E_limitedCapacityLinks;

		Map<Long, Pair<Long, Long>> linkMap = netPlan.getLinkMap();
		GraphTheoryMetrics metrics = new GraphTheoryMetrics(nodeIds, linkMap, null);
		Map<Long, Integer> nodeOutDegree = metrics.getOutNodeDegree();
		int[] maxMinOutDegree = nodeOutDegree.isEmpty() ? new int[] { 0, 0 } : IntUtils.maxMinValues(nodeOutDegree);
		double avgOutDegree = metrics.getAverageOutNodeDegree();
		
		Map<String, Object> topologyData = new LinkedHashMap<String, Object>();
		topologyData.put("Number of nodes", N);
		topologyData.put("Number of links", E);
		topologyData.put("Node out-degree (max, min, avg)", String.format("%d, %d, %.3f", maxMinOutDegree[0], maxMinOutDegree[1], avgOutDegree));
		topologyData.put("All links are bidirectional (yes/no)", GraphUtils.isBidirectional(linkMap) ? "Yes" : "No");
		
		if (applyHardComputations)
		{
			int networkDiameter_hops = (int) metrics.getDiameter();
			metrics.configureLinkCostMap(netPlan.getLinkLengthInKmMap());
			double networkDiameter_km = metrics.getDiameter();
			metrics.configureLinkCostMap(netPlan.getLinkPropagationDelayInSecondsMap());
			double networkDiameter_ms = 1000 * metrics.getDiameter();

			topologyData.put("Layer diameter (hops, km, ms)", String.format("%d, %.3f, %.3g", networkDiameter_hops, networkDiameter_km, networkDiameter_ms));
		}
		else
		{
			topologyData.put("Layer diameter (hops, km, ms)", "- (use 'Update all metrics' button)");
		}
		
		topologyData.put("Capacity installed: total", String.format("%.3f", U_e));
		topologyData.put("Capacity installed: average per link", String.format("%.3f", u_e_avg));
		topologyData.put("Capacity installed (limited capacity links): total", String.format("%.3f", totalCapacityInstalled_limitedCapacityLinks));
		topologyData.put("Capacity installed (limited capacity links): average per link", String.format("%.3f", averageTotalCapacityInstalled_limitedCapacityLinks));

		List<Map<String, Object>> layerSummaryInfo = new LinkedList<Map<String, Object>>();
		layerSummaryInfo.add(topologyData);
		
		Map<Long, Pair<Long, Long>> demandMap = netPlan.getDemandMap();
		Map<Long, Double> h_d = netPlan.getDemandOfferedTrafficMap();
		boolean isTrafficSymmetric = GraphUtils.isWeightedBidirectional(demandMap, h_d);
		double averageNodePairOfferedTraffic = netPlan.getNodePairAverageOfferedTraffic();
		double blockedTrafficPercentage = H_d == 0 ? 0 : 100 * (netPlan.getDemandTotalBlockedTraffic() / H_d);

		Map<String, Object> trafficData = new LinkedHashMap<String, Object>();
		trafficData.put("Number of demands", D);
		trafficData.put("Offered traffic: total", String.format("%.3f", H_d));
		trafficData.put("Offered traffic: average per node pair", String.format("%.3f", averageNodePairOfferedTraffic));
		trafficData.put("Blocked traffic (%)", String.format("%.3f", blockedTrafficPercentage));
		trafficData.put("Symmetric offered traffic?", isTrafficSymmetric ? "Yes" : "No");

		layerSummaryInfo.add(trafficData);
		
		double max_rho_e = netPlan.getLinkMaximumUtilization();
		RoutingType routingType = netPlan.getRoutingType();
		if (routingType == RoutingType.SOURCE_ROUTING)
		{
			double max_rho_e_noProtection = netPlan.getLinkMaximumUtilizationWithoutConsiderReservedCapacityForProtection();
			
			int R = netPlan.getNumberOfRoutes();
			int S = netPlan.getNumberOfProtectionSegments();
			boolean isRoutingBifurcated = netPlan.isRoutingBifurcated();
			boolean hasRoutingLoops = netPlan.hasRoutingLoops();

			double averageRouteLength_hops = TrafficComputationEngine.getRouteAverageLength(netPlan, null);
			double averageRouteLength_km = TrafficComputationEngine.getRouteAverageLength(netPlan, netPlan.getLinkLengthInKmMap());
			double averageRouteLength_ms = 1000 * TrafficComputationEngine.getRouteAverageLength(netPlan, netPlan.getLinkPropagationDelayInSecondsMap());

			Map<String, Object> routingData = new LinkedHashMap<String, Object>();
			routingData.put("Number of routes", R);
			routingData.put("Bifurcated routing (yes/no)", isRoutingBifurcated ? "Yes" : "No");
			routingData.put("Network congestion - bottleneck utilization (w. reserved bw, w.o. reserved bw)", String.format("%.3f, %.3f", max_rho_e, max_rho_e_noProtection));
			routingData.put("Average route length (hops, km, ms)", String.format("%.3f, %.3f, %.3g", averageRouteLength_hops, averageRouteLength_km, averageRouteLength_ms));
			routingData.put("Routing has loops?", hasRoutingLoops ? "Yes" : "No");

			layerSummaryInfo.add(routingData);

			Map<Long, Double> reservedCapacityMap = netPlan.getLinkCapacityReservedForProtectionMap();
			double u_e_reservedForProtection_avg = S == 0 ? 0 : DoubleUtils.sum(reservedCapacityMap) / S;
			double percentageReserved = 0;
			for(long linkId : linkIds)
			{
				double r_e = reservedCapacityMap.get(linkId);
				if (r_e == 0) continue;

				double u_e = netPlan.getLinkCapacity(linkId);
				percentageReserved += Math.max(0, r_e / u_e);
			}

			if (E != 0) percentageReserved *= 100.0 / E;

			Triple<Double, Double, Double> protectionPercentage = TrafficComputationEngine.getTrafficProtectionDegree(netPlan);
			double percentageUnprotected = protectionPercentage.getFirst();
			double percentageDedicated = protectionPercentage.getSecond();
			double percentageShared = protectionPercentage.getThird();
			Pair<Double, Double> srgDisjointnessPercentage = SRGUtils.getSRGDisjointnessPercentage(netPlan);
			String srgModel = SRGUtils.getSRGModel(netPlan);

			Map<String, Object> protectionData = new LinkedHashMap<String, Object>();
			protectionData.put("Number of protection segments in this layer", S);
			protectionData.put("Average link capacity reserved for protection (absolute, %)", String.format("%.3f, %.3f", u_e_reservedForProtection_avg, percentageReserved));
			protectionData.put("% of carried traffic unprotected", String.format("%.3f", percentageUnprotected));
			protectionData.put("% of carried traffic complete and dedicated protection", String.format("%.3f", percentageDedicated));
			protectionData.put("% of carried traffic partial and/or shared protection", String.format("%.3f", percentageShared));
			protectionData.put("Number of SRGs in the network", numSRGs);
			protectionData.put("SRG definition characteristic", srgModel);
			protectionData.put("% routes protected with SRG disjoint segments (w. end nodes, w.o. end nodes)", String.format("%.3f, %.3f", srgDisjointnessPercentage.getFirst(), srgDisjointnessPercentage.getSecond()));

			layerSummaryInfo.add(protectionData);
		}
		else
		{
			Map<String, Object> routingData = new LinkedHashMap<String, Object>();
			routingData.put("Network congestion - bottleneck utilization", String.format("%.3f", max_rho_e));
			
			RoutingCycleType routingCycleType = RoutingCycleType.LOOPLESS;
			for(long demandId : netPlan.getDemandIds())
			{
				RoutingCycleType routingCycleType_thisDemand = netPlan.getDemandRoutingCycleType(demandId);
				if (routingCycleType_thisDemand != RoutingCycleType.LOOPLESS) routingCycleType = routingCycleType_thisDemand;
				
				if (routingCycleType == RoutingCycleType.CLOSED_CYCLES) break;
			}
			
			routingData.put("Routing has loops?", routingCycleType);

			layerSummaryInfo.add(routingData);
		}
		
		ListIterator<Map<String, Object>> it = layerSummaryInfo.listIterator();
		while(it.hasNext())
		{
			int tableId = it.nextIndex();
			layerSummaryTables[tableId].setData(it.next());

			AdvancedJTable.setVisibleRowCount(layerSummaryTables[tableId], layerSummaryTables[tableId].getRowCount());
			AdvancedJTable.setWidthAsPercentages(layerSummaryTables[tableId], 0.7, 0.3);
		}

		layerSummaryTables[0].setToolTipText(0, 0, "Indicates the number of defined nodes in the network");
		layerSummaryTables[0].setToolTipText(1, 0, "Indicates the number of defined links in this layer");
		layerSummaryTables[0].setToolTipText(2, 0, "Indicates the maximum/minimum/average value for the out-degree, that is, the number of outgoing links per node");
		layerSummaryTables[0].setToolTipText(3, 0, "Indicates whether all links are bidirectional, that is, if there are the same number of links between each node pair in both directions (irrespective of the respective capacities)");
		layerSummaryTables[0].setToolTipText(4, 0, "Indicates the layer diameter, that is, the length of the largest shortest-path in this layer");
		layerSummaryTables[0].setToolTipText(5, 0, "Indicates the total capacity installed in this layer");
		layerSummaryTables[0].setToolTipText(6, 0, "Indicates the average capacity installed per link");
		layerSummaryTables[0].setToolTipText(7, 0, "Indicates the total capacity installed in this layer for links whose capacity is not infinite");
		layerSummaryTables[0].setToolTipText(8, 0, "Indicates the average capacity installed in this layer for links whose capacity is not infinite");
		layerSummaryTables[1].setToolTipText(0, 0, "Indicates the number of defined demands in this layer");
		layerSummaryTables[1].setToolTipText(1, 0, "Indicates the total offered traffic to the network in this layer");
		layerSummaryTables[1].setToolTipText(2, 0, "Indicates the total offered traffic to the network per each node pair in this layer");
		layerSummaryTables[1].setToolTipText(3, 0, "Indicates the percentage of blocked traffic from the total offered to the network in this layer");
		layerSummaryTables[1].setToolTipText(4, 0, "Indicates whether the offered traffic is symmetric, that is, if there are the same number of demands with the same offered traffic between each node pair in both directions");

		layerMetricsInfo.removeAll();
		layerMetricsInfo.add(new JLabel("Topology and link capacities"), "growx, wrap");
		layerMetricsInfo.add(new JScrollPane(layerSummaryTables[0]), "growx, wrap");
		layerMetricsInfo.add(new JLabel("Traffic"), "growx, wrap");
		layerMetricsInfo.add(new JScrollPane(layerSummaryTables[1]), "growx, wrap");
		
		if (routingType == RoutingType.SOURCE_ROUTING)
		{
			layerSummaryTables[2].setToolTipText(0, 0, "Indicates the number of defined routes");
			layerSummaryTables[2].setToolTipText(1, 0, "<html>Indicates whether the routing is bifurcated, that is, if at least there are more than one route <u>carrying</u> traffic from any demand</html>");
			layerSummaryTables[2].setToolTipText(2, 0, "Indicates the network congestion, that is, the utilization of the busiest link");
			layerSummaryTables[2].setToolTipText(3, 0, "Indicates the average route length");
			layerSummaryTables[2].setToolTipText(4, 0, "Indicates whether the routing has loops, that is, if at least a route visits a node more than once");
			layerSummaryTables[3].setToolTipText(0, 0, "Indicates the number of defined protection segments");
			layerSummaryTables[3].setToolTipText(1, 0, "Indicates the average reserved bandwidth per protection segment");
			layerSummaryTables[3].setToolTipText(2, 0, "Indicates the percentage of traffic (from the total) which is not protected by any segment");
			layerSummaryTables[3].setToolTipText(3, 0, "Indicates the percentage of traffic (from the total) which is protected by dedicated segments reservating the total traffic of their corresponding routes");
			layerSummaryTables[3].setToolTipText(4, 0, "Indicates the percentage of traffic (from the total) which is neither unprotected or dedicated");
			layerSummaryTables[3].setToolTipText(5, 0, "Indicates the number of defined SRGs");
			layerSummaryTables[3].setToolTipText(6, 0, "Indicates whether SRG definition follows one of the predefined models (per node, per link...), or 'Mixed' otherwise (or 'None' if no SRGs are defined)");
			layerSummaryTables[3].setToolTipText(7, 0, "Indicates the percentage of routes from the total which have all protection segments SRG-disjoint, without taking into account the carried traffic per route");

			layerMetricsInfo.add(new JLabel("Routing"), "growx, wrap");
			layerMetricsInfo.add(new JScrollPane(layerSummaryTables[2]), "growx, wrap");
			layerMetricsInfo.add(new JLabel("Resilience information"), "growx, wrap");
			layerMetricsInfo.add(new JScrollPane(layerSummaryTables[3]), "growx");
		}
		else
		{
			layerSummaryTables[2].setToolTipText(0, 0, "Indicates the network congestion, that is, the utilization of the busiest link");
			layerSummaryTables[2].setToolTipText(1, 0, "Indicates whether the routing has loops, that is, if at least a route visits a node more than once");

			layerMetricsInfo.add(new JLabel("Routing"), "growx, wrap");
			layerMetricsInfo.add(new JScrollPane(layerSummaryTables[2]), "growx, wrap");
		}
		
		forceUpdate.setVisible(!applyHardComputations);
	}
	
	/**
	 * Allows to show some custom log messages.
	 * 
	 * @param text Log message
	 * @since 0.3.0
	 */
	protected void updateLog(String text)
	{
	}
	
	private abstract static class DocumentAdapter implements DocumentListener
	{
		@Override
		public void changedUpdate(DocumentEvent e)
		{
			processEvent(e);
		}

		@Override
		public void insertUpdate(DocumentEvent e)
		{
			processEvent(e);
		}

		@Override
		public void removeUpdate(DocumentEvent e)
		{
			processEvent(e);
		}
		
		private void processEvent(DocumentEvent e)
		{
			Document doc = e.getDocument();
			try { updateInfo(doc.getText(0, doc.getLength())); }
			catch(BadLocationException ex) { }
		}

		protected abstract void updateInfo(String text);
	}

	private class SwitchTabAction extends AbstractAction
	{
		private final int tabId;
		
		public SwitchTabAction(int tabId)
		{
			this.tabId = tabId;
		}

		@Override
		public void actionPerformed(ActionEvent e)
		{
			showTab(tabId);
		}
	}
}
