/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.gui.utils.topology;

import com.net2plan.interfaces.networkDesign.NetPlan;
import com.net2plan.utils.Pair;

/**
 * Interface to be implemented by any class dealing with network designs.
 * 
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @since 0.2.3
 */
public interface INetworkCallback extends ITopologyCallback
{
	/**
	 * Returns the current network plan.
	 * 
	 * @return First item is the network plan, and the second one is the active layer
	 * @since 0.3.0
	 */
	public NetPlan getInitialDesign();

	/**
	 * Shows the given demand.
	 * 
	 * @param demandId Demand identifier
	 * @since 0.3.0
	 */
	public void showDemand(long demandId);

	/**
	 * Shows the given demand.
	 * 
	 * @param layerId Layer identifier
	 * @param demandId Demand identifier
	 * @since 0.3.0
	 */
	public void showDemand(long layerId, long demandId);

	/**
	 * Shows the given forwarding rule.
	 * 
	 * @param forwardingRule Forwarding rule identifier (first: demand identifier, second: link identifier)
	 * @since 0.3.0
	 */
	public void showForwardingRule(Pair<Long, Long> forwardingRule);

	/**
	 * Shows the given forwarding rule.
	 * 
	 * @param layerId Layer identifier
	 * @param forwardingRule Forwarding rule identifier (first: demand identifier, second: link identifier)
	 * @since 0.3.0
	 */
	public void showForwardingRule(long layerId, Pair<Long, Long> forwardingRule);

	/**
	 * Shows the given link.
	 * 
	 * @param layerId Layer identifier
	 * @param linkId Link identifier
	 * @since 0.3.0
	 */
	public void showLink(long layerId, long linkId);

	/**
	 * Shows the given node.
	 * 
	 * @param layerId Layer identifier
	 * @param nodeId Node identifier
	 * @since 0.3.0
	 */
	public void showNode(long layerId, long nodeId);

	/**
	 * Shows the given protection segment.
	 * 
	 * @param segmentId Protection segment identifier
	 * @since 0.3.0
	 */
	public void showProtectionSegment(long segmentId);

	/**
	 * Shows the given protection segment.
	 * 
	 * @param layerId Layer identifier
	 * @param segmentId Protection segment identifier
	 * @since 0.3.0
	 */
	public void showProtectionSegment(long layerId, long segmentId);

	/**
	 * Shows the given route.
	 * 
	 * @param routeId Route identifier
	 * @since 0.3.0
	 */
	public void showRoute(long routeId);

	/**
	 * Shows the given route.
	 * 
	 * @param layerId Layer identifier
	 * @param routeId Route identifier
	 * @since 0.3.0
	 */
	public void showRoute(long layerId, long routeId);

	/**
	 * Shows the given SRG.
	 * 
	 * @param srgId SRG identifier
	 * @since 0.3.0
	 */
	public void showSRG(long srgId);

	/**
	 * Shows the given SRG.
	 * 
	 * @param layerId Layer identifier
	 * @param srgId SRG identifier
	 * @since 0.3.0
	 */
	public void showSRG(long layerId, long srgId);

	/**
	 * Updates the layer chooser.
	 * 
	 * @since 0.3.0
	 */
	public void updateLayerChooser();
	
	/**
	 * Updates the <code>NetPlan</code> view (i.e. node info, link info, and so on).
	 * 
	 * @since 0.2.3
	 */
	public void updateNetPlanView();
	
	/**
	 * Updates the <code>NetPlan</code> warnings (over-subscribed links, blocked demands, and so on).
	 * 
	 * @since 0.3.0
	 */
	public void updateWarnings();
}
