/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.gui.utils.topology;

/**
 * Class representing a link.
 * 
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @since 0.2.0
 */
public class Link
{
	private final Node originNode;
	private final Node destinationNode;
	private long id;
	private String label;

	/**
	 * Default constructor.
	 * 
	 * @param id Link identifier
	 * @param originNode Origin node identifier
	 * @param destinationNode Destination node identifier
	 * @since 0.3.0
	 */
	public Link(long id, Node originNode, Node destinationNode)
	{
		this(id, originNode, destinationNode, null);
	}

	/**
	 * Constructor that allows to set a link label.
	 * 
	 * @param id Link identifier
	 * @param originNode Origin node identifier
	 * @param destinationNode Destination node identifier
	 * @param label Link label (may be null, default: "e" + link identifier)
	 * @since 0.3.0
	 */
	public Link(long id, Node originNode, Node destinationNode, String label)
	{
		this.id = id;
		this.originNode = originNode;
		this.destinationNode = destinationNode;

		this.label = label == null ? "e" + id : label;
	}

	@Override
	public String toString()
	{
		return getLabel();
	}

	/**
	 * Returns the destination node of the link.
	 * 
	 * @return Destination node
	 * @since 0.2.0
	 */
	public Node getDestinationNode()
	{
		return destinationNode;
	}

	/**
	 * Returns the link identifier.
	 * 
	 * @return Link identifier
	 * @since 0.3.0
	 */
	public long getId()
	{
		return id;
	}

	/**
	 * Returns the link label.
	 * 
	 * @return Link label
	 * @since 0.2.0
	 */
	public String getLabel()
	{
		return label;
	}
	
	/**
	 * Returns the origin node of the link.
	 * 
	 * @return Origin node
	 * @since 0.2.0
	 */
	public Node getOriginNode()
	{
		return originNode;
	}
}
