/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.gui.utils.topology.jung;

import com.net2plan.gui.utils.topology.ITopologyCallback;
import com.net2plan.gui.utils.topology.Link;
import com.net2plan.gui.utils.topology.Node;
import edu.uci.ics.jung.algorithms.layout.GraphElementAccessor;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * Plugin that enables to move nodes.
 * 
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @since 0.2.0
 */
public final class MoveNodePlugin extends AbstractCanvasPlugin implements MouseListener, MouseMotionListener
{
	private ITopologyCallback callback;
	private Node startVertex;

	/**
	 * Default constructor. Default modifier is "left button" click.
	 * 
	 * @param callback Topology callback listening plugin events
	 */
	public MoveNodePlugin(ITopologyCallback callback)
	{
		this(callback, MouseEvent.BUTTON1_MASK);
	}

	/**
	 * Constructor that allows to set the modifiers.
	 * 
	 * @param callback Topology callback listening plugin events
	 * @param modifiers Mouse event modifiers to activate this functionality
	 */
	public MoveNodePlugin(ITopologyCallback callback, int modifiers)
	{
		super(modifiers);

		this.callback = callback;
		startVertex = null;
	}

	@Override
	public void mouseClicked(MouseEvent e)
	{
		checkCanvas();
	}

	@Override
	public void mouseEntered(MouseEvent e)
	{
		checkCanvas();
	}

	@Override
	public void mouseExited(MouseEvent e)
	{
		checkCanvas();
	}

	@Override
	public void mousePressed(MouseEvent e)
	{
		checkCanvas();

		boolean accepted = checkModifiers(e);
		if (accepted == true)
		{
			final VisualizationViewer<Node, Link> vv = (VisualizationViewer<Node, Link>) e.getSource();
			final Point p = e.getPoint();
			GraphElementAccessor<Node, Link> pickSupport = vv.getPickSupport();
			if (pickSupport != null)
			{
				final Node vertex = pickSupport.getVertex(vv.getModel().getGraphLayout(), p.getX(), p.getY());

				if (vertex != null)
				{
					callback.showNode(vertex.getId());
					startVertex = vertex;
					e.consume();
				}
				else
				{
					callback.resetView();
				}
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e)
	{
		checkCanvas();

		startVertex = null;
	}

	@Override
	public void mouseDragged(MouseEvent e)
	{
		checkCanvas();

		if (startVertex != null)
		{
			Point p = e.getPoint();
			callback.moveNode(startVertex.getId(), getCanvas().convertViewCoordinatesToRealCoordinates(p));
			e.consume();
		}
	}

	@Override
	public void mouseMoved(MouseEvent e)
	{
		checkCanvas();
	}
}