/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.internal.io;

import com.net2plan.internal.plugins.IOFilter;
import com.net2plan.interfaces.networkDesign.NetPlan;
import com.net2plan.internal.Constants.IOFeature;
import com.net2plan.utils.DoubleUtils;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.util.EnumSet;
import java.util.Map;
import javax.swing.JComponent;

/**
 * This IO filter is targeted to handle traffic matrices stored in files where 
 * each line in the file is a row of the matrix. Items can be separated by 
 * spaces or tabs. Lines starting with '#' are skipped.
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @since 0.3.0
 */
public class IOTrafficMatrix2DFile extends IOFilter
{
	/**
	 * Default constructor.
	 *
	 * @since 0.3.0
	 */
	public IOTrafficMatrix2DFile()
	{
		super("2D traffic matrix", EnumSet.of(IOFeature.LOAD_DEMANDS, IOFeature.SAVE_DEMANDS), "txt", "csv");
	}
		
	@Override
	public JComponent getPluginConfigurationScreen()
	{
		return null;
	}

	@Override
	public Map<String, String> getPluginDefaultOptions()
	{
		return null;
	}

	@Override
	public String getPluginName()
	{
		return null;
	}

	@Override
	public NetPlan readDemandSetFromFile(File file)
	{
		double[][] trafficMatrix = DoubleUtils.read2DMatrixFromFile(file);
		NetPlan netPlan = new NetPlan();
		netPlan.setTrafficMatrix(trafficMatrix);
		
		return netPlan;
	}
	
	@Override
	public void saveDemandSetToFile(NetPlan netPlan, File file)
	{
		double[][] trafficMatrix = netPlan.getTrafficMatrix();
		int N = trafficMatrix.length;
		
		PrintStream out;
		try
		{
			out = new PrintStream(new FileOutputStream(file), true, StandardCharsets.UTF_8.name());
			out.println("# Number of nodes: " + N);
			for(int ingressNodeId = 0; ingressNodeId < N; ingressNodeId++)
			{
				for(int egressNodeId = 0; egressNodeId < N; egressNodeId++)
				{
					out.print(trafficMatrix[ingressNodeId][egressNodeId] + " ");
				}

				out.println();
			}
			
			out.flush();
			out.close();
		}
		catch(FileNotFoundException | UnsupportedEncodingException e)
		{
			throw new RuntimeException(e);
		}
	}
}
