/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.internal.plugins;

import com.net2plan.internal.SystemUtils;
import com.net2plan.utils.ClassLoaderUtils;
import java.io.File;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import javax.swing.filechooser.FileFilter;

/**
 *
 * @author Jose Luis
 */
public class PluginSystem
{
	private final static Map<Class<? extends Plugin>, Set<Class<? extends Plugin>>> plugins;
	
	static
	{
		plugins = new LinkedHashMap<Class<? extends Plugin>, Set<Class<? extends Plugin>>>();
		plugins.put(ICLIModule.class, new LinkedHashSet<Class<? extends Plugin>>());
		plugins.put(IGUIModule.class, new LinkedHashSet<Class<? extends Plugin>>());
		plugins.put(IOFilter.class, new LinkedHashSet<Class<? extends Plugin>>());
		plugins.put(ITopologyCanvas.class, new LinkedHashSet<Class<? extends Plugin>>());

		/* Load external plugins from 'plugins' folder */
		File pluginsFolder = new File(SystemUtils.getCurrentDir(), "plugins");
		if (pluginsFolder.exists() && pluginsFolder.isDirectory())
		{
			FileFilter fileFilter = ClassLoaderUtils.getFileFilter();

			for(File file : pluginsFolder.listFiles())
				if (file.isFile() && fileFilter.accept(file))
					for(Class<? extends Plugin> _class : plugins.keySet())
						for(Class<? extends Plugin> plugin : ClassLoaderUtils.getClassesFromFile(file, _class))
							addPlugin(_class, plugin);
		}
	}
	
	/**
	 * Adds an external plugin to the pluggable handler.
	 * 
	 * @param _class Class of the type of plugin implemented (GUI module, IO filter)
	 * @param plugin Class of the plugin
	 * @since 0.3.0
	 */
	public static void addPlugin(Class<? extends Plugin> _class, Class<? extends Plugin> plugin)
	{
		if (!plugins.containsKey(_class)) throw new RuntimeException("Bad - Invalid plugin type " + _class.getName());
		
		if (_class.isAssignableFrom(plugin)) plugins.get(_class).add(plugin);
		else throw new RuntimeException("Bad");
	}
	
	/**
	 * Returns the set of available plugins.
	 * 
	 * @param _class
	 * @return Set of available plugins
	 * @since 0.3.0
	 */
	public static Set<Class<? extends Plugin>> getPlugins(Class<? extends Plugin> _class)
	{
		return Collections.unmodifiableSet(plugins.get(_class));
	}
}